(*====================================================================
    File rconnect.pas

    Components library for Delphi, Kylix and BCB++.
    Login dialog.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rConnect.pas,v 1.5 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rConnect;

{$I POLARIS.INC}

interface

uses Windows, Classes, Graphics, Forms, Controls, StdCtrls,
     Buttons, ExtCtrls, SysUtils, ComCtrls, Registry, rConst, rButtons,
     RXCtrls, rSpeedButton;

type
  TrConnectAction = (caFail, caAbort, caRetry);
  TrConnectErrorEvent = procedure(Sender: TObject; E: Exception; var Action: TrConnectAction) of object;

  TrConnectCaptions = class(TPersistent)
  private
    FDialog, FResource, FUser, FPassword: TCaption;
  public
    constructor Create;
  published
    property Dialog: TCaption read FDialog write FDialog;
    property Resource: TCaption read FResource write FResource;
    property User: TCaption read FUser write FUser;
    property Password: TCaption read FPassword write FPassword;
  end;

  TrConnectHelp = class(TPersistent)
  private
    FDialog, FResource, FUser, FPassword, FOK, FCancel: THelpContext;
    FFile: string;
  public
    constructor Create;
  published
    property Dialog: THelpContext read FDialog write FDialog default 10800;
    property Resource: THelpContext read FResource write FResource default 10801;
    property User: THelpContext read FUser write FUser default 10802;
    property Password: THelpContext read FPassword write FPassword default 10803;
    property OK: THelpContext read FOK write FOK default 10804;
    property Cancel: THelpContext read FCancel write FCancel default 10805;
    property HelpFile: string read FFile write FFile;
  end;

  TrCustomConnect = class(TComponent)
  private
    FRegKey: string;
    FResourceEnabled: Boolean;
    FResourceDirectInput: Boolean;
    FResourceDataValue: string;
    FResources: TStringList;
    FUsers: TStringList;
    FUserEnabled: Boolean;
    FUserRequired: Boolean;
    FUserDataValue: string;
    FPasswordRequired: Boolean;
    FShowLastPassword: Boolean;
    FAutoLogon: Boolean;
    FMaxAttempts: Integer;
    FCaptions: TrConnectCaptions;
    FHelp: TrConnectHelp;
    FBeforeConnect: TNotifyEvent;
    FAfterConnect: TNotifyEvent;
    FOnError: TrConnectErrorEvent;
    FParams: TStrings;

    procedure SetRegKey(const Value: string);
    procedure SetCaptions(Value : TrConnectCaptions);
    procedure SetHelp(Value : TrConnectHelp);
    procedure SetResources(Value : TStringList);
    procedure SetUsers(Value : TStringList);
    function GetResource: String;
    procedure SetResource(Value: String);
    function GetUser: String;
    procedure SetUser(Value: String);
    function GetPassword: String;
    procedure SetPassword(Value: String);
    procedure SetParams(Value: TStrings);
  protected
    CountAttempts: Integer;
    property RegKey: string read FRegKey write SetRegKey;
    property ResourceEnabled: Boolean read FResourceEnabled write FResourceEnabled default True;
    property ResourceDirectInput: Boolean read FResourceDirectInput write FResourceDirectInput default False;
    property ResourceDataValue: string read FResourceDataValue write FResourceDataValue;
    property UserEnabled: Boolean read FUserEnabled write FUserEnabled default True;
    property UserRequired: Boolean read FUserRequired write FUserRequired default True;
    property UserDataValue: string read FUserDataValue write FUserDataValue;
    property Params: TStrings read FParams write SetParams;
    property Resource: String read GetResource write SetResource stored False;
    property User: String read GetUser write SetUser stored False;
    property Password: String read GetPassword write SetPassword stored False;
    property PasswordRequired: Boolean read FPasswordRequired write FPasswordRequired default True;
    property ShowLastPassword: Boolean read FShowLastPassword write FShowLastPassword default False;
    property AutoLogon: Boolean read FAutoLogon write FAutoLogon default False;
    property MaxAttempts: Integer read FMaxAttempts write FMaxAttempts default 3;
    property Captions: TrConnectCaptions read FCaptions write SetCaptions;
    property Help: TrConnectHelp read FHelp write SetHelp;
    property BeforeConnect: TNotifyEvent read FBeforeConnect write FBeforeConnect;
    property AfterConnect: TNotifyEvent read FAfterConnect write FAfterConnect;
    property OnError: TrConnectErrorEvent read FOnError write FOnError;
    procedure ConnectOperation; virtual;
    procedure DoAfterConnect; virtual;
    procedure DoBeforeConnect; virtual;
    procedure DoConnect; virtual;
    function EnableConnect: Boolean; virtual;
  public
    property Resources: TStringList read FResources write SetResources;
    property Users: TStringList read FUsers write SetUsers;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Connect: Boolean; virtual;
  end;

  TrConnect = class(TrCustomConnect)
  private
    FOnConnect: TNotifyEvent;
  public
    procedure DoConnect; override;
  published
    property Params;
    property RegKey;
    property Resources;
    property Resource;
    property ResourceEnabled;
    property ResourceDirectInput;
    property User;
    property UserEnabled;
    property UserRequired;
    property Password;
    property PasswordRequired;
    property ShowLastPassword;
    property AutoLogon;
    property MaxAttempts;
    property Captions;
    property Help;
    property BeforeConnect;
    property AfterConnect;
    property OnError;
    property OnConnect: TNotifyEvent read FOnConnect write FOnConnect;
  end;

  TrPasswordDlg = class(TForm)
    OKBtn: TrBitBtn;
    CancelBtn: TBitBtn;
    Bevel1: TBevel;
    BitBtn1: TBitBtn;
    GroupBox1: TGroupBox;
    LabelResource: TLabel;
    CBResource: TComboBox;
    EditUser: TComboBox;
    LabelUser: TLabel;
    LabelPass: TLabel;
    EditPassword: TEdit;
    rSpeedButton1: TrSpeedButton;
    Animate1: TAnimate;
    Image1: TImage;
    procedure CBResourceChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditPasswordChange(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    function FormHelp(Command: Word; Data: Integer;
      var CallHelp: Boolean): Boolean;
    procedure rSpeedButton1Click(Sender: TObject);
  private
  public
  end;

var
  PasswordDlg: TrPasswordDlg;

implementation

uses rUtils, rVCLUtils {$IFNDEF POLARIS_D4}, StrUtils{$ENDIF};

{$R *.DFM}

{ TrConnectCaptions }

constructor TrConnectCaptions.Create;
begin
  FDialog := srConnectDialog;
  FResource := srConnectResource;
  FUser := srConnectUser;
  FPassword := srConnectPassword;
end;

{ TrConnectHelp }

constructor TrConnectHelp.Create;
begin
  FDialog   := 10800;
  FResource := 10801;
  FUser     := 10802;
  FPassword := 10803;
  FOK       := 10804;
  FCancel   := 10805;
  FFile     := srHelpFile;
end;

{ TrCustomConnect }

constructor TrCustomConnect.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FParams := TStringList.Create;
  FCaptions := TrConnectCaptions.Create;
  FHelp := TrConnectHelp.Create;
  FRegKey := '';
  FResources := TStringList.Create;
  FUsers := TStringList.Create;
  FMaxAttempts := 3;
  FResourceEnabled := True;
  FResourceDirectInput := False;
  FResourceDataValue := srResourceValue;
  FUserEnabled := True;
  FUserRequired := True;
  FUserDataValue := srUserValue;
  FPasswordRequired := True;
  FAutoLogon := False;
end;

destructor TrCustomConnect.Destroy;
begin
  FCaptions.Free;
  FHelp.Free;
  FResources.Free;
  FUsers.Free;
  FParams.Free;
  inherited Destroy;
end;

procedure TrCustomConnect.SetRegKey;
var
  r: HKEY;
  s: String;
begin
  if FRegKey <> Value then begin
    FRegKey := Value;
    with TRegistry.Create do
      try
        SeparateRegKey(FRegKey, r, s);
        RootKey := r;
        if OpenKey(s, False) then 
          try
            if ValueExists(FResourceDataValue) then
              Resource := ReadString(FResourceDataValue)
            else Resource := '';
            if ValueExists(FUserDataValue) then
{$IFDEF POLARIS_D4}
              FUsers.Text := StringReplace(ReadString(FUserDataValue), ';', #13#10, [rfReplaceAll])
{$ELSE}
              FUsers.Text := ReplaceStr(ReadString(FUserDataValue), ';', #13#10)
{$ENDIF}
            else
              FUsers.Clear;
{            if FUsers.Count = 0 then FUser := ''
            else FUser := FUsers.Strings[0];}
            if FUsers.Count = 0 then User := ''
            else User := FUsers.Strings[0];
          finally
            CloseKey;
          end;
      finally
        Free;
      end;
  end;
end;

procedure TrCustomConnect.SetCaptions(Value : TrConnectCaptions);
begin
  FCaptions.Assign(Value);
end;

procedure TrCustomConnect.SetHelp(Value : TrConnectHelp);
begin
  FHelp.Assign(Value);
end;

procedure TrCustomConnect.SetResources(Value : TStringList);
begin
  FResources.Assign(Value);
end;

procedure TrCustomConnect.SetUsers(Value : TStringList);
begin
  FUsers.Assign(Value);
end;

function TrCustomConnect.GetResource: String;
begin
  Result := FParams.Values[srpResourceName];
end;

procedure TrCustomConnect.SetResource(Value: String);
begin
  if Value <> FParams.Values[srpResourceName]
  then FParams.Values[srpResourceName] := Value;
end;

function TrCustomConnect.GetUser: String;
begin
  Result := FParams.Values[srpUserName];
end;

procedure TrCustomConnect.SetUser(Value: String);
begin
  if Value <> FParams.Values[srpUserName]
  then FParams.Values[srpUserName] := Value;
end;

function TrCustomConnect.GetPassword: String;
begin
  Result := FParams.Values[srpPassword]
end;

procedure TrCustomConnect.SetPassword(Value: String);
begin
  if Value <> FParams.Values[srpPassword]
  then FParams.Values[srpPassword] := Value;
end;

procedure TrCustomConnect.SetParams(Value: TStrings);
begin
  FParams.Assign(Value);
end;

procedure TrCustomConnect.DoConnect;
begin
  Inc(CountAttempts);
end;

procedure TrCustomConnect.ConnectOperation;
var
  Done: Boolean;
  Action: TrConnectAction;
begin
  DoBeforeConnect;
  Done := False;
  repeat
    try
      DoConnect;
      Done := True;
    except
      on E: Exception do
      begin
        Action := caFail;
        if Assigned(FOnError) then FOnError(Self, E, Action);
        if Action = caFail then raise;
        if Action = caAbort then SysUtils.Abort;
      end;
    end;
  until Done;
  DoAfterConnect;
end;

procedure TrCustomConnect.DoAfterConnect;
begin
  if Assigned(FAfterConnect) then FAfterConnect(Self);
end;

procedure TrCustomConnect.DoBeforeConnect;
begin
  if Assigned(FBeforeConnect) then FBeforeConnect(Self);
end;

function TrCustomConnect.Connect: Boolean;
var
  s, oldResource, oldUser, oldPassword: string;
  i: Integer;
  r: HKEY;
begin
  CountAttempts := 0;
  Result := False;
  if FAutoLogon then begin
    ConnectOperation;
    Result := True;
  end;
  if not Result then
    with TrPasswordDlg.Create(Self) do
      try
        oldResource := Resource;
        oldUser := User;
        oldPassword := Password;

        Caption := FCaptions.FDialog;
        HelpFile := FHelp.FFile;
        HelpContext := FHelp.FDialog;
        LabelResource.Caption := FCaptions.FResource;
        CBResource.HelpContext := FHelp.FResource;
        LabelUser.Caption := FCaptions.FUser;
        EditUser.HelpContext := FHelp.FUser;
        LabelPass.Caption := FCaptions.FPassword;
        EditPassword.HelpContext := FHelp.FPassword;
        OKBtn.HelpContext := FHelp.FOK;
        CancelBtn.HelpContext := FHelp.FCancel;

        with CBResource do begin
          Enabled := FResourceEnabled;
          Items.Assign(FResources);
          if ResourceDirectInput then begin
            Style := csDropDown;
            Text := Resource;
          end else
            ItemIndex := Items.IndexOf(Resource);
          if (Text = '') and (Items.Count > 0) then ItemIndex := 0;
          Resource := Text;
        end;
        EditUser.Enabled := FUserEnabled;
        rSpeedButton1.Enabled := FUserEnabled;
        EditUser.Items.Assign(FUsers);
        EditUser.Text := User;
        if FShowLastPassword then EditPassword.Text := Password;
        Result := (ShowModal = mrOK);
      finally
        if not Result then begin
          Resource := oldResource;
          User := oldUser;
          Password := oldPassword;
        end;
        Free;
      end;
  if Result then
    if FRegKey <> '' then
      with TRegistry.Create do
        try
          SeparateRegKey(FRegKey, r, s);
          RootKey := r;
          if OpenKey(s, True) then
            try
              s := '';
              WriteString(FResourceDataValue, Resource);

              i := FUsers.IndexOf(User);
              if i < 0 then
                s := User + ';'
              else
                if i <> 0 then FUsers.Exchange(i,0);
{$IFDEF POLARIS_D4}
              s := s + StringReplace(FUsers.Text, #13#10, ';', [rfReplaceAll]);
{$ELSE}
              s := s + ReplaceStr(FUsers.Text, #13#10, ';');
{$ENDIF}
              WriteString(FUserDataValue, s);
            finally
              CloseKey;
            end;
        finally
          Free;
        end;
end;

function TrCustomConnect.EnableConnect: Boolean;
begin
  Result := (Resource <> '') and ((User <> '') or not FUserRequired) and
            ((Password <> '') or not FPasswordRequired);
end;

{ TrConnect }

procedure TrConnect.DoConnect;
begin
  inherited;
  if Assigned(FOnConnect) then FOnConnect(Self);
end;

{ TrPasswordDlg }

procedure TrPasswordDlg.CBResourceChange(Sender: TObject);
begin
  if Owner is TrCustomConnect then begin
    TrCustomConnect(Owner).Resource := CBResource.Text;
    EditUser.OnChange(EditUser);
  end;
end;

procedure TrPasswordDlg.FormCreate(Sender: TObject);
begin
  Image1.Picture.Bitmap.LoadFromResourceName(hInstance, srConnectRes);
end;

procedure TrPasswordDlg.EditPasswordChange(Sender: TObject);
begin
  if Owner is TrCustomConnect then
    with TrCustomConnect(Owner) do begin
      if Assigned(Sender) then
        if Sender = EditUser then User := EditUser.Text
        else Password := EditPassword.Text
      else begin
        User := EditUser.Text;
        Password := EditPassword.Text;
      end;
      OKBtn.Enabled := EnableConnect;
    end;
end;

procedure TrPasswordDlg.OKBtnClick(Sender: TObject);
var
  avifn: TFileName;
begin
  if Owner is TrCustomConnect then begin
    avifn := ExtractFilePath(GetModulePath(0)) + 'rconnect.avi';   // don't localize
    if FileExists(avifn) then
      Animate1.FileName := avifn
    else
      if not use_wine then Animate1.CommonAVI := aviFindComputer;
    Animate1.Show;
    Animate1.Active := not use_wine;
    with TrCustomConnect(Owner) do
      try
        ConnectOperation;
        ModalResult := mrOK;
      except
        if CountAttempts = MaxAttempts then
          ModalResult := mrCancel
        else begin
          Animate1.Active := False;
          Animate1.Hide;
          if (EditUser.Text = '') and EditUser.Enabled then begin
            EditUser.SelectAll;
            EditUser.SetFocus;
          end else begin
            EditPassword.SelectAll;
            EditPassword.SetFocus;
          end;
        end;
        raise;
      end;
  end;
end;

procedure TrPasswordDlg.FormActivate(Sender: TObject);
begin
  if CBResource.Text = '' then
    ActiveControl := CBResource
  else if (EditUser.Text <> '') or (not EditUser.Enabled) then
    ActiveControl := EditPassword
  else
    ActiveControl := EditUser;
  EditUser.OnChange(nil);
end;

function TrPasswordDlg.FormHelp(Command: Word; Data: Integer;
  var CallHelp: Boolean): Boolean;
begin
  CallHelp := not ((Command = HELP_CONTEXTPOPUP) and (Data = HelpContext));
  Result := True;
end;

procedure TrPasswordDlg.rSpeedButton1Click(Sender: TObject);
var
  i: Integer;
begin
  with EditUser do begin
    i := Items.IndexOf(Text);
    if i >= 0 then begin
      Items.Delete(i);
      if Self.Owner is TrCustomConnect then
        (Self.Owner as TrCustomConnect).Users.Text := Items.Text;
    end;
    if Items.Count > 0 then Text := Items[0]
    else Text := '';
    EditUser.OnChange(EditUser);
  end;
end;

end.

