(*====================================================================
    File rDBXconnect.pas

    Components library for Delphi, Kylix and BCB++.
    Connect to database with DBX.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDBXConnect.pas,v 1.5 2008/10/06 11:45:47 khmelevskiy Exp $
======================================================================*)
unit rDBXConnect;

{$I POLARIS.INC}

interface

uses
  Classes, SysUtils, rConnect, rDBConst, sqlExpr, DBConnAdmin, SqlConst, rConst,
  rDBXUtils {$IFDEF POLARIS_D10}, WideStrings{$ENDIF};

type
  TrDBXConnect = class(TrCustomConnect)
  private
    FDBConnection: TSQLConnection;
    FDrivers: string;
    procedure SetDrivers(const Value: string);
    function GetConnectionName: string;
    procedure SetConnectionName(const Value: string);
    function GetConnectionNameEnabled: Boolean;
    procedure SetConnectionNameEnabled(const Value: Boolean);
    procedure SetSQLConnection(Value: TSQLConnection);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DoConnect; override;
  published
    property Params;
    property DBConnection: TSQLConnection read FDBConnection write SetSQLConnection;
    property Drivers: string read FDrivers write SetDrivers;
    property ConnectionName: string read GetConnectionName write SetConnectionName;
    property ConnectionNameEnabled: Boolean read GetConnectionNameEnabled write SetConnectionNameEnabled default True;
    property RegKey;
    property User;
    property UserEnabled;
    property UserRequired;
    property Password;
    property PasswordRequired;
    property ShowLastPassword;
    property AutoLogon;
    property MaxAttempts;
    property Captions;
    property Help;
    property BeforeConnect;
    property AfterConnect;
    property OnError;
  end;

implementation

uses {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils{$ENDIF};

{ TrDBXConnect }

constructor TrDBXConnect.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ConnectionNameEnabled := True;
  Captions.Resource := srConnectDatabase;
  ResourceDataValue := srDatabaseValue;
  with Help do begin
    Dialog   := 10200;
    Resource := 10201;
    User     := 10202;
    Password := 10203;
    OK       := 10204;
    Cancel   := 10205;
  end;
  GetConnectionNames(Resources);
end;

procedure TrDBXConnect.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDBConnection) then
    DBConnection := nil;
end;

procedure TrDBXConnect.SetSQLConnection(Value: TSQLConnection);
begin
  if FDBConnection <> Value then begin
    FDBConnection := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TrDBXConnect.SetDrivers(const Value: string);
begin
  if FDrivers <> Value then begin
    FDrivers := Value;
    GetConnectionNames(Resources, FDrivers);
  end;
end;

function TrDBXConnect.GetConnectionName: string;
begin
  Result := Resource;
end;

procedure TrDBXConnect.SetConnectionName(const Value: string);
var
  paramList:
  {$IFDEF POLARIS_D12}TStringList
  {$ELSE}
    {$IFDEF POLARIS_D10} TWideStringList
    {$ELSE} TStringList{$ENDIF}
  {$ENDIF};
  ConnectionAdmin: IConnectionAdmin;
begin
  if Resource <> Value then begin
    Resource := Value;
    if (User = '') then begin
      paramList := {$IFDEF POLARIS_D12}TStringList.Create;
  {$ELSE}
    {$IFDEF POLARIS_D10} TWideStringList.Create;
    {$ELSE} TStringList.Create;{$ENDIF}
  {$ENDIF};
      try
        ConnectionAdmin := GetConnectionAdmin;
        ConnectionAdmin.GetConnectionParams(Value, paramList);
        User := AnsiUpperCase(paramList.Values[szUSERNAME]);
      finally
        paramList.Free;
      end;
    end;
  end;
end;

function TrDBXConnect.GetConnectionNameEnabled: Boolean;
begin
  Result := ResourceEnabled;
end;

procedure TrDBXConnect.SetConnectionNameEnabled(const Value: Boolean);
begin
  ResourceEnabled := Value;
end;

procedure TrDBXConnect.DoConnect;
begin
  inherited;
  with FDBConnection do begin
    Close;
    ConnectionName := '';
    DriverName := '';
    Params.Text := srpResourceName + '=' + Self.ConnectionName;
    UpdateDBXDriverParams(FDBConnection);
    LoginPrompt := False;
    Params.Values[szUSERNAME] := User;
    Params.Values[szPASSWORD] := Password;
    Open;
  end;
end;

end.

