(*====================================================================
    File rDSCSV.pas

    Components library for Delphi and BCB++.
    rDSCSV component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDSCSV.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rDSCSV;

{$I POLARIS.INC}

interface
uses SysUtils, Classes, DB, rUtils,
     {$IFDEF POLARIS_D6} RxStrUtils {$ELSE} StrUtils {$ENDIF};

type

  TCSVConvert=(csvcANSI, csvcOEM);
  TCSVProgressEvent = procedure (Sender: TObject; Position: Integer) of object;
  TrGetDataSetProc = function (aName: string): TDataSet of object;

  TrDSCSV = class (TComponent)
  private
    FBeforeExport: TNotifyEvent;
    FOnProgress: TCSVProgressEvent;
    FSeparator: Char;
    FDelimiter: Char;
    FFileName: string;
    FSeparatorUse,
    FExportFieldTitle,
    FExportOnlyVisible: Boolean;
    FProgressSteps,
    FProgress: Integer;
    FConvertType: TCSVConvert;
    FOnGetDataSet: TrGetDataSetProc;
    procedure SetFileName(Value: string);
    function GetDataSet(Index: Integer): TDataSet;
    function GetDataSetCount: Integer;
    function GetPercent: Integer;
  protected
    procedure CalcMaxStep;
    procedure IncProgress;
  public
    AllDataSets: TStringList;
    //****
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Convert;
    property DataSet[Index: Integer]: TDataSet read GetDataSet;
    property DataSetCount: Integer read GetDataSetCount;
    property Percent: Integer read GetPercent;
  published
    property BeforeExport: TNotifyEvent read FBeforeExport write FBeforeExport;
    property OnProgress: TCSVProgressEvent read FOnProgress write FOnProgress;
    property Separator: Char read FSeparator write FSeparator;
    property SepartorUse: Boolean read FSeparatorUse write FSeparatorUse default False;
    property Delimiter: Char read FDelimiter write FDelimiter;
    property FileName: string read FFileName write SetFileName;
    property ExportFieldTitle: Boolean read FExportFieldTitle write FExportFieldTitle default False;
    property ExportOnlyVisible: Boolean read FExportOnlyVisible write FExportOnlyVisible default True;
    property ConvertType: TCSVConvert read FConvertType write FConvertType default csvcANSI;
    property OnGetDataSet: TrGetDataSetProc read FOnGetDataSet write FOnGetDataSet;
  end;

implementation

{ TrDSCSV }

procedure TrDSCSV.CalcMaxStep;
var
  I: Integer;
  D: TDataSet;
begin
  FProgressSteps := 0;
  for I := 0 to DataSetCount-1 do begin
    D := DataSet[I];
    if (D <> nil) and D.Active
    then
      if D.IsUniDirectional then
        Inc(FProgressSteps,1) else
        Inc(FProgressSteps,D.RecordCount);
    if FProgressSteps = MaxInt then break;
  end;
end;

procedure TrDSCSV.Convert;

const
  OKFields: Set of TFieldType = [ftString, ftSmallint, ftWord, ftFloat, ftInteger, ftCurrency,
                                 ftBCD, ftFMTBCD, ftDate, ftTime, ftDateTime, ftMemo, ftTimeStamp];

  function GetTitleString(DS: TDataSet): string;
  var
    I: Integer;
    S: string;
  begin
    Result := '';
    for I := 0 to DS.FieldCount-1 do
    if (not FExportOnlyVisible or DS.Fields[I].Visible) and (DS.Fields[I].DataType in OKFields)
    then begin
      S := DS.Fields[I].DisplayLabel;
      if FSeparatorUse and (FSeparator<>#0)
        then S := FSeparator+S+FSeparator;
      {$ifdef mswindows}
      if FConvertType <> csvcANSI
        then S := StrToOem(S);
      {$endif}
      if Result = ''
        then Result := S
        else Result := Result+FDelimiter+S;
    end;
  end;

  function GetDataString(DS: TDataSet): string;
  var
    I: Integer;
    S: string;
    IsFirstColumn: boolean;
  begin
    Result := '';
    IsFirstColumn := true;
    for I := 0 to DS.FieldCount-1 do
      if (not FExportOnlyVisible or DS.Fields[I].Visible) and (DS.Fields[I].DataType in OKFields) then
        begin
          S := DS.Fields[I].AsString;
          case DS.Fields[I].DataType of
            ftBCD, ftFMTBCD,
            ftCurrency,
            ftFloat:    S := ReplaceStr(S, DecimalSeparator,'.');
            ftDate,
            ftTime,
            ftDateTime,
            ftTimeStamp: if not DS.Fields[I].IsNull then
                            S := DateTimeToStrFix(DS.Fields[I].AsDateTime);
          end;
          S := ReplaceStr(ReplaceStr(S, ^M, ''), ^J, '');
          if FSeparatorUse and (FSeparator <> #0) then S := FSeparator + S + FSeparator;
	  {$ifdef mswindows}
          if FConvertType <> csvcANSI then S := StrToOem(S);
	  {$endif}
          //if Result = '' then Result := S else Result := Result+FDelimiter+S;
          if IsFirstColumn then Result := S else Result := Result+FDelimiter+S;
          IsFirstColumn := false;
        end;
  end;

var
  DS: TDataSet;
  ResStream: TFileStream;
  L,
  I: Integer;
  S: PChar;
begin
//***
  if FFileName = '' then Exit;
  if Assigned(fBeforeExport) then fBeforeExport(Self);

  CalcMaxStep;
//***
  ResStream := TFileStream.Create(FFileName, fmCreate or fmOpenReadWrite or fmShareDenyWrite);
  try
    for I := 0 to DataSetCount - 1 do begin
      DS := DataSet[I];
      if (DS = nil) and Assigned(FOnGetDataSet) then DS := FOnGetDataSet(AllDatasets.Strings[i]);
      if DS <> nil then begin
        DS.DisableControls;
        try
          if DS.Active and not (DS.BOF and DS.EOF) then begin
            if FExportFieldTitle then begin
              S := PChar(GetTitleString(DS)+^M^J);
              L := Length(S);
              ResStream.Write(S^, L);
            end;
           if not DS.IsUniDirectional then DS.First;
            while not DS.EOF do begin
              S := PChar(GetDataString(DS)+^M^J);
              L := Length(S);
              ResStream.Write(S^, L);
              DS.Next;
              IncProgress;
            end;
          end;
        finally
          DS.EnableControls;
        end;
      end;
    end;
  finally
    ResStream.Free;
    FProgress := -1;
    IncProgress;
  end;
end;

constructor TrDSCSV.Create(AOwner: TComponent);
begin
  inherited;
  FConvertType   := csvcANSI;
  FProgressSteps := 0;
  FProgress      := 0;
  FSeparator     := #0;
  FDelimiter     := ';';
  FSeparatorUse  := False;
  FBeforeExport  := nil;
  FOnProgress    := nil;
  AllDataSets    := TStringList.Create;
  FileName       := '';
  FExportFieldTitle := False;
  FExportOnlyVisible := True;
  FProgressSteps := 0;
  FProgress      := 0;
end;

destructor TrDSCSV.Destroy;
begin
  AllDataSets.Free;
  inherited;
end;

function TrDSCSV.GetDataSet(Index: Integer): TDataSet;
begin
  Result := nil;
  if ((Index < AllDatasets.Count) and (Index >= 0))
  and (AllDatasets.Objects[Index] <> nil)
  then
    Result := AllDatasets.Objects[Index] as TDataSet;
end;

function TrDSCSV.GetDataSetCount: Integer;
begin
  Result := AllDatasets.Count;
end;

function TrDSCSV.GetPercent: Integer;
begin
  Result := 0;
  if FProgressSteps <> 0
  then Result := Round(FProgress/FProgressSteps*100);
end;

procedure TrDSCSV.IncProgress;
var
  Per1, Per2: Integer;
begin
  Per1 := Percent;
  Inc(FProgress);
  Per2 := Percent;
  if (Per1 <> Per2) or (Per2 = 0) then begin
    if Assigned(FOnProgress) then FOnProgress(Self, Per2);
  end;
end;

procedure TrDSCSV.SetFileName(Value: string);
begin
  if Value <> FFileName
  then FFileName := Value;
end;

end.
