(*====================================================================
    File rExtrnDS.pas

    Components library for Delphi and BCB++.
    External DataSet server & client components

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rExtrnDS.pas,v 1.4 2008/10/06 11:58:10 khmelevskiy Exp $
======================================================================*)

unit rExtrnDS;

{$I POLARIS.INC}

interface

uses
  {$IFDEF MSWINDOWS} Windows, ActiveX, {$ENDIF}
  Types, Classes, SysUtils, DB, rConst, rUtils, DBConsts, RTLConsts,
  MaxMin, RxStrUtils;

const
  ecNone             =  0;
  ecOpen             =  1;
  ecClose            =  2;
  ecFirst            =  3;
  ecLast             =  4;
  ecPrior            =  5;
  ecNext             =  6;
  ecBOF              =  7;
  ecEOF              =  8;
  ecIsUnidirectional =  9;
  ecIsEmpty          = 10;
  ecCanModify        = 11;
  ecGetFieldData     = 12;
  ecGetRecordCount   = 13;
  ecGetBlobData      = 14;

type

  TrExternalDataProc = function (cmd, n: DWORD; p: Pointer): Pointer;
  {$IFDEF MSWINDOWS} stdcall; {$ENDIF}
  {$IFDEF LINUX}     cdecl;   {$ENDIF}
  TrExternalDataProcObj = function (Sender: TObject; cmd, n: DWORD; p: Pointer): Cardinal of object;

  prFieldTypeSize = ^TrFieldTypeSize;
  TrFieldTypeSize = record
    FType: TFieldType;
    FSize: Integer;
  end;
  TrGetDataFieldTypeProc = function (FieldName: string): Cardinal of object;

  prExternalFieldDef = ^TrExternalFieldDef;
  TrExternalFieldDef = packed record
    FName: array[1..50] of Char;
    FDisplay: array[1..100] of Char;
    FType: DWORD;
    FSize: DWORD;
    FVisible: LongBool;
    FTag: DWORD;
    FLast: DWORD;
  end;

//==============================================

  TrCustomDataServer = class(TComponent)
  public
    function ExecuteCommand(Sender: TObject; aCmd, aN: DWORD; aP: Pointer): Cardinal; virtual; abstract;
  end;

  TrExternalDataServer = class(TrCustomDataServer)
  private
    FDataSet: TDataSet;
    FBol: Boolean;
    FDWORD: DWORD;
  public
    function ExecuteCommand(Sender: TObject; aCmd, aN: DWORD; aP: Pointer): Cardinal; override;
  published
    property DataSet: TDataSet read FDataSet write FDataSet;
  end;

  TFieldTypesList = class(TStringList)
  public
    procedure AddField(aName: string; aTypeSize: TrFieldTypeSize);
    procedure Clear; override;
    destructor Destroy; override;
  end;

  TrTextFileDataServer = class(TrCustomDataServer)
  private
    FFileName: TFileName;
    FFile: TextFile;
    FDelimiter: Char;
    FFieldCount: Integer;
    FList: TFieldTypesList;
    FGetDataFieldTypeProc: TrGetDataFieldTypeProc;
    FFileEOF, FActive: Boolean;
    //
    FCurrentRecord: string;
    FBol: Boolean;
    FDWORD: DWORD;
    FFloat: Double;
    FString: string;
    procedure ReadNextLine;
    function GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    function ExecuteCommand(Sender: TObject; aCmd, aN: DWORD; aP: Pointer): Cardinal; override;
    property FieldCount: Integer read FFieldCount;
    destructor Destroy; override;
  published
    property FileName: TFileName read FFileName write FFileName;
    property Delimiter: Char read FDelimiter write FDelimiter stored True;
    property GetDataFieldTypeProc: TrGetDataFieldTypeProc read FGetDataFieldTypeProc write FGetDataFieldTypeProc;
  end;

//==============================================

  TrExternalDataSet = class(TDataSet)
  private
    FExternalDataProc: TrExternalDataProc;
    FExternalDataProcObj: TrExternalDataProcObj;
    FDataServer: TrCustomDataServer;
    function CallExternal(cmd, n: DWord; p: Pointer): Pointer;
    function CalcFieldNo(aField: TField): Integer;
  protected
    procedure InternalOpen; override;
    procedure InternalClose; override;
    function IsCursorOpen: Boolean; override;
    function GetCanModify: Boolean; override;
    function GetRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF}; GetMode: TGetMode; DoCheck: Boolean): TGetResult; override;
    procedure InitRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF}); override;
    procedure SetBufListSize(Value: Integer); override;
    function GetRecordCount: Integer; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
  public
    constructor Create(AOwner: TComponent); override;
    function GetFieldData(Field: TField; Buffer: Pointer): Boolean; overload; override;
    function GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean; overload; override;
    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream; override;
    function IsSequenced: Boolean; override;
    property ExternalDataProc: TrExternalDataProc read FExternalDataProc write FExternalDataProc;
  published
    property ExternalDataProcObj: TrExternalDataProcObj read FExternalDataProcObj write FExternalDataProcObj;
    property DataServer: TrCustomDataServer read FDataServer write FDataServer;
  end;

//==============================================
implementation

{$ifdef POLARIS_D6}
  {$WARN SYMBOL_PLATFORM OFF}
{$endif}

function __GlobalGetMem(aSize: Integer): Pointer;
begin
  {$IFDEF MSWINDOWS}
  Result := GlobalAllocPtr(HeapAllocFlags or GMEM_ZEROINIT, aSize);
  {$ELSE}
  GetMem(Result, aSize);
  {$ENDIF}
end;

procedure __GlobalFreeMem(aP: Pointer);
begin
  {$IFDEF MSWINDOWS}
  GlobalFreePtr(aP);
  {$ELSE}
  FreeMem(aP);
  {$ENDIF}
end;

//==============================================
//
// TrExternalDataServer
//
//==============================================

function TrExternalDataServer.ExecuteCommand(Sender: TObject; aCmd, aN: DWORD; aP: Pointer): Cardinal;
var
  i: DWORD;
  p: prExternalFieldDef;
  s: string;
  f: TField;
  w: WideString;
begin
  Result := 0;
  case aCmd of
    ecOpen: begin
      if not Assigned(FDataSet) then raise Exception.Create(srNoExtDataSet);
      with FDataSet do begin
        if not Active then Open;
        p := __GlobalGetMem(FieldCount * SizeOf(TrExternalFieldDef));
        Result := DWORD(p);
        for i := 0 to FieldCount - 1 do begin
          with p^ do begin
            StrPCopy(@FName, Fields[i].FieldName);
            StrPCopy(@FDisplay, Copy(Fields[i].DisplayLabel, 1, 100));
            FType := iif(Fields[i] is TWideStringField, Ord(ftString), Ord(Fields[i].DataType));
            FSize := iif(Fields[i] is TWideStringField, Fields[i].Size, Fields[i].DataSize);
            FVisible := Fields[i].Visible;
            FTag := Fields[i].Tag;
            FLast := iif(Integer(i) < Fields.Count - 1, 0, 1);
          end;
          Inc(p);
        end;
        First;
      end;
    end;
    ecClose: begin
      FDataSet.Close;
    end;
    ecFirst: FDataSet.First;
    ecLast: FDataSet.Last;
    ecPrior: FDataSet.Prior;
    ecNext: FDataSet.Next;
    ecBOF: begin
      FBol := FDataSet.BOF;
      Result := DWORD(@FBol);
    end;
    ecEOF: begin
      FBol := FDataSet.EOF;
      Result := DWORD(@FBol);
    end;
    ecIsUnidirectional: begin
      FBol := FDataSet.IsUnidirectional;
      Result := DWORD(@FBol);
    end;
    ecIsEmpty: begin
      FBol := FDataSet.IsEmpty;
      Result := DWORD(@FBol);
    end;
    ecCanModify: begin
      FBol := FDataSet.CanModify;
      Result := DWORD(@FBol);
    end;
    ecGetFieldData: begin
      f := FDataSet.Fields[aN];
      if f is TWideStringField then begin
        FBol := (aP <> nil) and not f.IsNull;
        if FBol then begin
          PChar(aP)[f.Size] := #0;
          w := TWideStringField(f).Value;
          {$ifdef MSWINDOWS}
          WideCharToMultiByte(0, 0, pWideChar(w), SysStringLen(pWideChar(w)) + 1, aP, f.Size, nil, nil);
          {$endif}
        end;
      end
      else
        FBol := FDataSet.GetFieldData(f, aP);
      Result := DWORD(@FBol);
    end;
    ecGetRecordCount: begin
      FDWORD := FDataSet.RecordCount;
      Result := DWORD(@FDWORD);
    end;
    ecGetBlobData: begin
      FDWORD := 0;
      if (aP <> nil) and (FDataSet.Fields[aN].DataType in [ftBlob, ftMemo, ftGraphic, ftFmtMemo]) then begin
        s := FDataSet.Fields[aN].AsString;
        FDWORD := Length(s);
        if FDWORD > 0 then begin
          pPointer(aP)^ := __GlobalGetMem(FDWORD + 1);
          Move(s[1], pPointer(aP)^^, FDWORD + 1);
        end;
      end;
      Result := DWORD(@FDWORD);
    end;
    else raise Exception.CreateFmt(srUnknownCommandCode, [aCmd]);
  end;
end;

//==============================================
//
// TrTextFileDataServer
//
//==============================================

procedure TFieldTypesList.AddField(aName: string; aTypeSize: TrFieldTypeSize);
var
  p: prFieldTypeSize;
begin
  New(p);
  p^ := aTypeSize;
  AddObject(aName, Pointer(p));
end;

procedure TFieldTypesList.Clear;
var
  i: Integer;
begin
  for i := 0 to Count - 1 do Dispose(prFieldTypeSize(Objects[i]));
  inherited;
end;

destructor TFieldTypesList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

constructor TrTextFileDataServer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActive := False;
  FDelimiter := #9;
  FList := TFieldTypesList.Create;
end;

procedure TrTextFileDataServer.ReadNextLine;
begin
  if FFileEOF then FCurrentRecord := ''
  else begin
    FFileEOF := EOF(FFile);
    if not FFileEOF then begin
      repeat
        ReadLn(FFile, FCurrentRecord);
      until EOF(FFile) or (FCurrentRecord <> '');
      FFileEOF := FCurrentRecord = '';
    end;
  end;
end;

function TrTextFileDataServer.ExecuteCommand(Sender: TObject; aCmd, aN: DWORD; aP: Pointer): Cardinal;
var
  w: string;
  i: DWORD;
  p: prExternalFieldDef;

  procedure DoLoadError;
  begin
    CloseFile(FFile);
    raise Exception.CreateFmt(SFOpenError, [FFileName]);
  end;

begin
  Result := 0;
  case aCmd of
    ecOpen: begin
      FList.Clear;
      if not Assigned(FGetDataFieldTypeProc) then raise Exception.Create(srNoGetDataTypeProc);
      if FFileName = '' then raise Exception.Create(srNoExtDataSet);
      if not FileExists(FFileName) then raise Exception.CreateFmt(SInvalidFileName, [FFileName]);
      AssignFile(FFile, FFileName);
      Reset(FFile);
      FActive := True;
      FFileEOF := EOF(FFile);
      if FFileEOF then DoLoadError;
      ReadNextLine;
      if FCurrentRecord = '' then DoLoadError;
      FFieldCount := WordCount(FCurrentRecord, [FDelimiter]);
      FList.Clear;
      p := __GlobalGetMem(FieldCount * SizeOf(TrExternalFieldDef));
      Result := DWORD(p);
      for i := 1 to FieldCount do begin
        w := ExtractWord(i, FCurrentRecord, [FDelimiter]);
        FList.AddField(w, prFieldTypeSize(FGetDataFieldTypeProc(w))^);
        with p^ do begin
          StrPCopy(@FName, w);
          FType := ord(prFieldTypeSize(FList.Objects[i - 1]).FType);
          FSize := prFieldTypeSize(FList.Objects[i - 1]).FSize;
          FLast := iif(Integer(i) < FieldCount, 0, 1);
        end;
        inc(p);
      end;
      ReadNextLine;
    end;
    ecClose: if FActive then begin
      {$I-}
      CloseFile(FFile);
      {$I+}
      FActive := False;
    end;
    ecFirst: ;
    ecLast: ;
    ecPrior: ;
    ecNext: ReadNextLine;
    ecBOF: begin
      FBol := False;
      Result := DWORD(@FBol);
    end;
    ecEOF: begin
      FBol := FFileEOF;
      Result := DWORD(@FBol);
    end;
    ecIsUnidirectional: begin
      FBol := True;
      Result := DWORD(@FBol);
    end;
    ecIsEmpty: begin
      FBol := EOF(FFile);
      Result := DWORD(@FBol);
    end;
    ecCanModify: begin
      FBol := False;
      Result := DWORD(@FBol);
    end;
    ecGetFieldData: begin
      FBol := GetFieldData(aN + 1, aP);
      Result := DWORD(@FBol);
    end;
    ecGetRecordCount: begin
      FDWORD := $FFFFFFFF;
      Result := DWORD(@FDWORD);
    end
    else raise Exception.CreateFmt(srUnknownCommandCode, [aCmd]);
  end;
end;

function TrTextFileDataServer.GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean;
begin
  FString := ExtractDelimited(FieldNo, FCurrentRecord, [FDelimiter]);
  Result := FString <> '';
  if Result and Assigned(Buffer) then begin
    case prFieldTypeSize(FList.Objects[FieldNo - 1]).FType of
      ftString: Move(PChar(FString)^, Buffer^, Min(
        dsMaxStringSize, Min(
        prFieldTypeSize(FList.Objects[FieldNo - 1]).FSize + 1,
        Length(FString) + 1)));
      ftInteger: begin
        FDWORD := StrToIntDef(FString, 0);
        Move(FDWORD, Buffer^, SizeOf(DWORD));
      end;
      ftFloat: begin
        FFloat := StrToFloatFix(FString);
        Move(FFloat, Buffer^, SizeOf(Double));
      end;
      ftBoolean: begin
        FBol := FString = '1';
        Move(FBol, Buffer^, SizeOf(Boolean));
      end;
      ftDate: begin
        FDWORD := DateTimeToTimeStamp(StrToDateFix(FString)).Date;
        Move(FDWORD, Buffer^, SizeOf(FDWORD));
      end;
      ftTime: begin
        FDWORD := DateTimeToTimeStamp(StrToTimeFix(FString)).Time;
        Move(FDWORD, Buffer^, SizeOf(FDWORD));
      end;
      ftDateTime: begin
        FFloat := TimeStampToMSecs(DateTimeToTimeStamp(StrToDateTimeFix(FString)));
        Move(FFloat, Buffer^, SizeOf(Double));
      end
      else Result := False;
    end;
  end;
end;

destructor TrTextFileDataServer.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

//==============================================
//
// TrExternalDataSet
//
//==============================================

constructor TrExternalDataSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetUniDirectional(True);
  ObjectView := False;
end;

function TrExternalDataSet.CallExternal(cmd, n: DWORD; p: Pointer): Pointer;
begin
  if Assigned(FExternalDataProcObj) then
    DWORD(Result) := FExternalDataProcObj(Self, cmd, n, p)
  else
  if Assigned(FExternalDataProc) then
    Result := FExternalDataProc(cmd, n, p)
  else
  if Assigned(FDataServer) then
    DWORD(Result) := FDataServer.ExecuteCommand(Self, cmd, n, p)
  else
    raise Exception.Create(srNoExtDataProc);
end;

procedure TrExternalDataSet.InternalOpen;
var
  p0: pointer;
  p: prExternalFieldDef;
  vComplete: Boolean;
  i, sz: Integer;
begin
  p0 := CallExternal(ecOpen, ecNone, nil);
  if not Assigned(p0) then raise Exception.Create(SRecordNotFound);
  try

    p := p0;
    repeat
      case TFieldType(p.FType) of
        ftString: sz := p.FSize
        else sz := 0;
      end;
      FieldDefs.Add(PChar(@p.FName), TFieldType(p.FType), sz, False);
      vComplete := Boolean(p.FLast);
      inc(p);
    until vComplete;

    CreateFields;
    BindFields(True);

    i := 0;
    p := p0;
    repeat
      Fields[i].DisplayLabel := PChar(@p.FDisplay);
      Fields[i].Visible := p.FVisible;
      Fields[i].Tag := p.FTag;
      vComplete := Boolean(p.FLast);
      inc(i);
      inc(p);
    until vComplete;

  finally
    __GlobalFreeMem(p0);
  end;
end;

procedure TrExternalDataSet.InternalClose;
begin
  CallExternal(ecClose, ecNone, nil);
  Fields.Clear;
  FieldDefs.Clear;
end;

function TrExternalDataSet.IsCursorOpen: Boolean;
begin
  Result := True;
end;

function TrExternalDataSet.GetCanModify: Boolean;
begin
  Result := False;
end;

function TrExternalDataSet.GetRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF}; GetMode: TGetMode; DoCheck: Boolean): TGetResult;
begin
  if not BOF then CallExternal(ecNext, ecNone, nil);
  if pBoolean(CallExternal(ecEOF, ecNone, nil))^
    then Result := grEOF
    else Result := grOK;
end;

function TrExternalDataSet.CalcFieldNo(aField: TField): Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to Fields.Count - 1 do
    if aField.FieldName = Fields[i].FieldName then begin
      Result := i + 1;
      Break;
    end;
end;

function TrExternalDataSet.GetFieldData(Field: TField; Buffer: Pointer): Boolean;
var
  n: Integer;
begin
  Result := False;
  n := CalcFieldNo(Field);
  if n > 0 then Result := GetFieldData(n, Buffer);
end;

function TrExternalDataSet.GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean;
begin
  Result := pBoolean(CallExternal(ecGetFieldData, FieldNo - 1, Buffer))^;
end;

function TrExternalDataSet.CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
var
  s: string;
  sz: Integer;
  p: PChar;
begin
  s := '';
  sz := CalcFieldNo(Field);
  if sz > 0 then begin
    sz := pInteger(CallExternal(ecGetBlobData, sz - 1, @p))^;
    if sz > 0 then begin
      s := p;
      __GlobalFreeMem(p);
    end;
  end;
  Result := TStringStream.Create(s);
end;

procedure TrExternalDataSet.InitRecord(Buffer: {$IFDEF POLARIS_D12}PByte{$ELSE}PChar{$ENDIF});
begin
  // nop
end;

procedure TrExternalDataSet.SetBufListSize(Value: Integer);
begin
  // nop
end;

function TrExternalDataSet.IsSequenced: Boolean;
begin
  Result := False;
end;

function TrExternalDataSet.GetRecordCount: Integer;
begin
  Result := pInteger(CallExternal(ecGetRecordCount, 0, nil))^;
end;

procedure TrExternalDataSet.InternalHandleException;
begin
  // nop
end;

procedure TrExternalDataSet.InternalInitFieldDefs;
begin
  // nop
end;

//==============================================
end.

