(*====================================================================
    File rIBQuery.pas

    Components library for Delphi and BCB++.
    rIBQuery component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rIBQuery.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rIBQuery;

interface


uses
  Windows, Messages, SysUtils, Classes, DB, IBCustomDataSet, IBQuery,
  Work_with_Params
  {$IFDEF RSQLMONITOR} ,DDEMan {$ENDIF}
  ;

////    ExecSQL,   

type
  TrIBQuery = class(TIBQuery)
  private
    {$IFDEF RSQLMONITOR}
    FDDE: TDDEClientConv;
    {$ENDIF}
    FOldSQLChange:TNotifyEvent;
    FMacroChar: Char;
    FMacros: TParams;
    function  GetMacros: TParams;
    procedure SetMacros(Value: TParams);
    function  GetMacroCount: Word;
    procedure SetMacroChar(Value: Char);
    procedure RecreateMacros;
    procedure CreateMacros(List: TParams; const Value: PChar);
    procedure NewSQLChange( Sender:TObject );
    function GetRealSQL:String;
  protected
    procedure DoBeforeOpen; override;
    procedure InternalOpen; override;
    procedure InternalExecQuery; override;
  public
    constructor Create( AOwner:TComponent ); override;
    destructor Destroy; override;
    function MacroByName(const Value: string): TParam;
    function IsEmpty: Boolean;
    procedure Post; override;
    procedure Cancel; override;
    procedure ExecSQL;
    property MacroCount: Word read GetMacroCount;
    property RealSQL:String read GetRealSQL;
  published
    property MacroChar: Char read FMacroChar write SetMacroChar default DefaultMacroChar;
    property Macros: TParams read GetMacros write SetMacros;
  end;

procedure Register;

implementation
{$IFDEF RSQLMONITOR}
Uses Work_with_SQLMonitor;
{$ENDIF}


procedure Register;
begin
  RegisterComponents('MiDB', [TrIBQuery]);
end;




{ TrIBQuery }

constructor TrIBQuery.Create(AOwner: TComponent);
begin
  inherited;
  {$IFDEF RSQLMONITOR}
  FDDE := CheckDDE( Self );
  {$ENDIF}
  FMacroChar := DefaultMacroChar;
  FMacros := TParams.Create;
  FOldSQLChange := TStringList( SQL ).OnChange;
  TStringList( SQL ).OnChange := NewSQLChange;
end;

destructor TrIBQuery.Destroy;
begin
  FMacros.Free;
  inherited;
end;

function TrIBQuery.GetMacroCount: Word;
begin
  Result := FMacros.Count;
end;

function TrIBQuery.GetMacros: TParams;
begin
  Result := FMacros;
end;

function TrIBQuery.IsEmpty: Boolean;
begin
  Result := not Active or (BOF and EOF);
end;

function TrIBQuery.MacroByName(const Value: string): TParam;
begin
  Result := FMacros.ParamByName(Value);
end;

procedure TrIBQuery.RecreateMacros;
var
  List: TParams;
begin
    List := TParams.Create;
    try
      CreateMacros(List, PChar(SQL.Text));
      List.AssignValues(FMacros);
      FMacros.Free;
      FMacros := List;
    except
      List.Free;
    end;
end;

procedure TrIBQuery.SetMacroChar(Value: Char);
begin
  if Value <> FMacroChar then
    begin
      FMacroChar := Value;
      RecreateMacros;
    end;
end;

procedure TrIBQuery.SetMacros(Value: TParams);
begin
  FMacros.AssignValues(Value);
end;

procedure TrIBQuery.CreateMacros(List: TParams; const Value: PChar);
begin
  CreateQueryParams(List, Value, True, MacroChar, ['.']);
end;


procedure TrIBQuery.NewSQLChange(Sender: TObject);
begin
  if Assigned( FOldSQLChange ) then FOldSQLChange( Sender );
  RecreateMacros;
end;

function TrIBQuery.GetRealSQL:String;

  function ReplaceString(const S: string): string;
  var
    I, J, P, LiteralChars: Integer;
    Param: TParam;
    Found: Boolean;
  begin
    Result := S;
    for I := Macros.Count - 1 downto 0 do begin
      Param := Macros[I];
      if Param.DataType = ftUnknown then Continue;
      repeat
        P := Pos(MacroChar + Param.Name, Result);
        Found := (P > 0) and ((Length(Result) = P + Length(Param.Name)) or
          NameDelimiter(Result[P + Length(Param.Name) + 1], ['.']));
        if Found then begin
          LiteralChars := 0;
          for J := 1 to P - 1 do
            if IsLiteral(Result[J]) then Inc(LiteralChars);
          Found := LiteralChars mod 2 = 0;
          if Found then begin
            Result := Copy(Result, 1, P - 1) + Param.Text + Copy(Result,
              P + Length(Param.Name) + 1, MaxInt);
          end;
        end;
      until not Found;
    end;
  end;

var
  I: Integer;
begin
  Result := '';
  for I := 0 to SQL.Count - 1 do
    Result := Result + ReplaceString(SQL[I]) + #13#10;
  Result := copy( Result, 1, length(Result) - 2 );
end;

procedure TrIBQuery.DoBeforeOpen;
begin
  SelectSQL.Text := RealSQL;
  inherited;
end;

procedure TrIBQuery.Cancel;
begin
  inherited;
end;

procedure TrIBQuery.Post;
begin
  inherited;
end;



procedure TrIBQuery.InternalOpen;
{$IFDEF RSQLMONITOR}
Var LastTime: TDateTime;
    Executed: boolean;
{$ENDIF}
begin
  {$IFDEF RSQLMONITOR}
    Executed := false;
    LastTime := Now;
    try
      inherited;
      Executed := true;
      LastTime := Now - LastTime;
      OutToSQLMonitor( FDDE, '', LastTime, Name, SQL.Text, RealSQL, Macros, Params );
    except
      on E: Exception do
        begin
          if not Executed then
            begin
              LastTime := Now - LastTime;
              OutToSQLMonitor( FDDE, E.Message, LastTime, Name, SQL.Text, RealSQL, Macros, Params );
            end;
          raise;
        end;
    end;  
  {$ELSE}
    inherited;
  {$ENDIF}
end;

procedure TrIBQuery.InternalExecQuery;
{$IFDEF RSQLMONITOR}
Var LastTime: TDateTime;
    Executed: boolean;
{$ENDIF}
begin
  {$IFDEF RSQLMONITOR}
    Executed := false;
    LastTime := Now;
    try
      inherited;
      Executed := true;
      LastTime := Now - LastTime;
      OutToSQLMonitor( FDDE, '', LastTime, Name, SQL.Text, RealSQL, Macros, Params );
    except
      on E: Exception do
        begin
          if not Executed then
            begin
              LastTime := Now - LastTime;
              OutToSQLMonitor( FDDE, E.Message, LastTime, Name, SQL.Text, RealSQL, Macros, Params );
            end;
          raise;
        end;
    end;  
  {$ELSE}
    inherited;
  {$ENDIF}
end;


procedure TrIBQuery.ExecSQL;
begin
  SelectSQL.Text := RealSQL;
  TIBQuery( Self ).ExecSQL;
end;

end.
