(*====================================================================
    File rLinux.pas

    Components library for Delphi and BCB++.
        Linux.
    Linux specific functions and types.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rLinux.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

{
   Copyright (C) 1991,1994-2002,2003,2004 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
}


unit rLinux;

interface

uses Types, Libc;

type

  bfd_format = (
    bfd_unknown,	{ File format is unknown.  }
    bfd_object,		{ Linker/assembler/compiler output.  }
    bfd_archive,	{ Object archive file.  }
    bfd_core,		{ Core dump.  }
    bfd_type_end	{ Marks the end; don't use it!  }
  );

  pbfd = ^bfd;
  bfd = packed record
  end;

  pbfd_section = ^bfd_section;
  bfd_section = packed record

    { The name of the section; the name isn't a copy, the pointer is
       the same as that passed to bfd_make_section.  }
    name: PChar;

    { A unique sequence number.  }
    id: Integer;

    { Which section in the bfd; 0..n-1 as sections are created in a bfd.  }
    index: Integer;

    { The next section in the list belonging to the BFD, or NULL.  }
    next: pbfd_section;

    { The field flags contains attributes of the section. Some
       flags are read in from the object file, and some are
       synthesized from other information.  }
    flags: DWORD;

  //#define SEC_NO_FLAGS   0x000

    { Tells the OS to allocate space for this section when loading.
       This is clear for a section containing debug information only.  }
  //#define SEC_ALLOC      0x001

    { Tells the OS to load the section from the file when loading.
       This is clear for a .bss section.  }
  //#define SEC_LOAD       0x002

    { The section contains data still to be relocated, so there is
       some relocation information too.  }
  //#define SEC_RELOC      0x004

    { ELF reserves 4 processor specific bits and 8 operating system
       specific bits in sh_flags; at present we can get away with just
       one in communicating between the assembler and BFD, but this
       isn't a good long-term solution.  }
  //#define SEC_ARCH_BIT_0 0x008

    { A signal to the OS that the section contains read only data.  }
  //#define SEC_READONLY   0x010

    { The section contains code only.  }
  //#define SEC_CODE       0x020

    { The section contains data only.  }
  //#define SEC_DATA       0x040

    { The section will reside in ROM.  }
  //#define SEC_ROM        0x080

    { The section contains constructor information. This section
       type is used by the linker to create lists of constructors and
       destructors used by <<g++>>. When a back end sees a symbol
       which should be used in a constructor list, it creates a new
       section for the type of name (e.g., <<__CTOR_LIST__>>), attaches
       the symbol to it, and builds a relocation. To build the lists
       of constructors, all the linker has to do is catenate all the
       sections called <<__CTOR_LIST__>> and relocate the data
       contained within - exactly the operations it would peform on
       standard data.  }
  //#define SEC_CONSTRUCTOR 0x100

    { The section has contents - a data section could be
       <<SEC_ALLOC>> | <<SEC_HAS_CONTENTS>>; a debug section could be
       <<SEC_HAS_CONTENTS>>  }
  //#define SEC_HAS_CONTENTS 0x200

    { An instruction to the linker to not output the section
       even if it has information which would normally be written.  }
  //#define SEC_NEVER_LOAD 0x400

    { The section is a COFF shared library section.  This flag is
       only for the linker.  If this type of section appears in
       the input file, the linker must copy it to the output file
       without changing the vma or size.  FIXME: Although this
       was originally intended to be general, it really is COFF
       specific (and the flag was renamed to indicate this).  It
       might be cleaner to have some more general mechanism to
       allow the back end to control what the linker does with
       sections.  }
  //#define SEC_COFF_SHARED_LIBRARY 0x800

    { The section contains thread local data.  }
  //#define SEC_THREAD_LOCAL 0x1000

    { The section has GOT references.  This flag is only for the
       linker, and is currently only used by the elf32-hppa back end.
       It will be set if global offset table references were detected
       in this section, which indicate to the linker that the section
       contains PIC code, and must be handled specially when doing a
       static link.  }
  //#define SEC_HAS_GOT_REF 0x4000

    { The section contains common symbols (symbols may be defined
       multiple times, the value of a symbol is the amount of
       space it requires, and the largest symbol value is the one
       used).  Most targets have exactly one of these (which we
       translate to bfd_com_section_ptr), but ECOFF has two.  }
  //#define SEC_IS_COMMON 0x8000

    { The section contains only debugging information.  For
       example, this is set for ELF .debug and .stab sections.
       strip tests this flag to see if a section can be
       discarded.  }
  //#define SEC_DEBUGGING 0x10000

    { The contents of this section are held in memory pointed to
       by the contents field.  This is checked by bfd_get_section_contents,
       and the data is retrieved from memory if appropriate.  }
  //#define SEC_IN_MEMORY 0x20000

    { The contents of this section are to be excluded by the
       linker for executable and shared objects unless those
       objects are to be further relocated.  }
  //#define SEC_EXCLUDE 0x40000

    { The contents of this section are to be sorted based on the sum of
       the symbol and addend values specified by the associated relocation
       entries.  Entries without associated relocation entries will be
       appended to the end of the section in an unspecified order.  }
  //#define SEC_SORT_ENTRIES 0x80000

    { When linking, duplicate sections of the same name should be
       discarded, rather than being combined into a single section as
       is usually done.  This is similar to how common symbols are
       handled.  See SEC_LINK_DUPLICATES below.  }
  //#define SEC_LINK_ONCE 0x100000

    { If SEC_LINK_ONCE is set, this bitfield describes how the linker
       should handle duplicate sections.  }
  //#define SEC_LINK_DUPLICATES 0x600000

    { This value for SEC_LINK_DUPLICATES means that duplicate
       sections with the same name should simply be discarded.  }
  //#define SEC_LINK_DUPLICATES_DISCARD 0x0

    { This value for SEC_LINK_DUPLICATES means that the linker
       should warn if there are any duplicate sections, although
       it should still only link one copy.  }
  //#define SEC_LINK_DUPLICATES_ONE_ONLY 0x200000

    { This value for SEC_LINK_DUPLICATES means that the linker
       should warn if any duplicate sections are a different size.  }
  //#define SEC_LINK_DUPLICATES_SAME_SIZE 0x400000

    { This value for SEC_LINK_DUPLICATES means that the linker
       should warn if any duplicate sections contain different
       contents.  }
  //#define SEC_LINK_DUPLICATES_SAME_CONTENTS 0x600000

    { This section was created by the linker as part of dynamic
       relocation or other arcane processing.  It is skipped when
       going through the first-pass output, trusting that someone
       else up the line will take care of it later.  }
  //#define SEC_LINKER_CREATED 0x800000

    { This section should not be subject to garbage collection.  }
  //#define SEC_KEEP 0x1000000

    { This section contains "short" data, and should be placed
       "near" the GP.  }
  //#define SEC_SMALL_DATA 0x2000000

    { This section contains data which may be shared with other
       executables or shared objects.  }
  //#define SEC_SHARED 0x4000000

    { When a section with this flag is being linked, then if the size of
       the input section is less than a page, it should not cross a page
       boundary.  If the size of the input section is one page or more, it
       should be aligned on a page boundary.  }
  //#define SEC_BLOCK 0x8000000

    { Conditionally link this section; do not link if there are no
       references found to any symbol in the section.  }
  //#define SEC_CLINK 0x10000000

    { Attempt to merge identical entities in the section.
       Entity size is given in the entsize field.  }
  //#define SEC_MERGE 0x20000000

    { If given with SEC_MERGE, entities to merge are zero terminated
       strings where entsize specifies character size instead of fixed
       size entries.  }
  //#define SEC_STRINGS 0x40000000

    { This section contains data about section groups.  }
  //#define SEC_GROUP 0x80000000

    {  End of section flags.  }

    { Some internal packed boolean fields.  }

    // 1-bit fields:

    { See the vma field.  }
    //user_set_vma: DWORD; // bit 0

    { Whether relocations have been processed.  }
    //reloc_done: DWORD; // bit 1

    { A mark flag used by some of the linker backends.  }
    //linker_mark: DWORD; // bit 2

    { Another mark flag used by some of the linker backends.  Set for
       output sections that have an input section.  }
    //linker_has_input: DWORD; // bit 3

    { A mark flag used by some linker backends for garbage collection.  }
    //gc_mark: DWORD; // bit 4

    { The following flags are used by the ELF linker. }

    { Mark sections which have been allocated to segments.  }
    //segment_mark: DWORD; // bit 5

    { Type of sec_info information.  }
    //sec_info_type: DWORD; // 3 bits: 6-8
  //#define ELF_INFO_TYPE_NONE      0
  //#define ELF_INFO_TYPE_STABS     1
  //#define ELF_INFO_TYPE_MERGE     2
  //#define ELF_INFO_TYPE_EH_FRAME  3
  //#define ELF_INFO_TYPE_JUST_SYMS 4

    { Nonzero if this section uses RELA relocations, rather than REL.  }
    //use_rela_p: DWORD; // bit 9

    { Bits used by various backends.  }
    //has_tls_reloc: DWORD; // bit 10

    { Nonzero if this section needs the relax finalize pass.  }
    //need_finalize_relax: DWORD; // bit 11

    { Nonzero if this section has a gp reloc.  }
    //has_gp_reloc: DWORD; // bit 12

    { Unused bits.  }
    //flag13: DWORD; // bit 13
    //flag14: DWORD; // bit 14
    //flag15: DWORD; // bit 15

    // stub1: fields user_set_vma - flag15
    stub1: WORD;

    //flag16: DWORD; // 4 bits
    //flag20: DWORD; // 4 bits
    //flag24: DWORD; // 8 bits

    // stub2: fields flag16 - flag24
    stub2: WORD;

    { End of internal packed boolean fields.  }

    {  The virtual memory address of the section - where it will be
        at run time.  The symbols are relocated against this.  The
        user_set_vma flag is maintained by bfd; if it's not set, the
        backend can assign addresses (for example, in <<a.out>>, where
        the default address for <<.data>> is dependent on the specific
        target and various flags).  }
    vma: Int64;

    {  The load address of the section - where it would be in a
        rom image; really only used for writing section header
        information.  }
    lma: Int64;

    { The size of the section in octets, as it will be output.
       Contains a value even if the section has no contents (e.g., the
       size of <<.bss>>).  This will be filled in after relocation.  }
    _cooked_size: Int64;

    { The original size on disk of the section, in octets.  Normally this
       value is the same as the size, but if some relaxing has
       been done, then this value will be bigger.  }
    _raw_size: Int64;

    { If this section is going to be output, then this value is the
       offset in *bytes* into the output section of the first byte in the
       input section (byte ==> smallest addressable unit on the
       target).  In most cases, if this was going to start at the
       100th octet (8-bit quantity) in the output section, this value
       would be 100.  However, if the target byte size is 16 bits
       (bfd_octets_per_byte is "2"), this value would be 50.  }
    output_offset: Int64;

    { The output section through which to map on output.  }
    output_section: pbfd_section;

    { The alignment requirement of the section, as an exponent of 2 -
       e.g., 3 aligns to 2^3 (or 8).  }
    alignment_power: DWORD;

    { If an input section, a pointer to a vector of relocation
       records for the data in this section.  }
    relocation: Pointer;

    { If an output section, a pointer to a vector of pointers to
       relocation records for the data in this section.  }
    orelocation: Pointer;

    { The number of relocation records in one of the above.  }
    reloc_count: DWORD;

    { Information below is back end specific - and not always used
       or updated.  }

    { File position of section data.  }
    filepos: Int64;

    { File position of relocation info.  }
    rel_filepos: Int64;

    { File position of line data.  }
    line_filepos: Int64;

    { Pointer to data for applications.  }
    userdata: Pointer;

    { If the SEC_IN_MEMORY flag is set, this points to the actual
       contents.  }
    contents: Pointer;

    { Attached line number information.  }
    lineno: Pointer;

    { Number of line number records.  }
    lineno_count: DWORD;

    { Entity size for merging purposes.  }
    entsize: DWORD;

    { Optional information about a COMDAT entry; NULL if not COMDAT.  }
    comdat: Pointer;

    { Points to the kept section if this section is a link-once section,
       and is discarded.  }
    kept_section: pbfd_section;

    { When a section is being output, this value changes as more
       linenumbers are written out.  }
    moving_line_filepos: Int64;

    { What the section number is in the target world.  }
    target_index: Integer;

    used_by_bfd: Pointer;

    { If this is a constructor section then here is a list of the
       relocations created to relocate items within it.  }
    constructor_chain: Pointer;

    { The BFD which owns the section.  }
    owner: pbfd;

    { A symbol which points at this section only.  }
    symbol: Pointer;
    symbol_ptr_ptr: Pointer;

    link_order_head: Pointer;
    link_order_tail: Pointer;

  end; // bfd_section

const rBFDLibraryName = 'libbfd.so';

procedure bfd_init(); cdecl; external rBFDLibraryName;
function bfd_openr(const filename: PChar; const target: PChar): pbfd; cdecl; external rBFDLibraryName;
function bfd_check_format(abfd: pbfd; format: {bfd_format}DWORD): Boolean; cdecl; external rBFDLibraryName;
function bfd_get_section_by_name(abfd: pbfd; const name: PChar): pbfd_section; cdecl; external rBFDLibraryName;
function bfd_get_section_contents(abfd: pbfd; section: pbfd_section; location: Pointer; offset: Int64; count: Int64): Boolean; cdecl; external rBFDLibraryName;
function bfd_close(abfd: pbfd): Boolean; cdecl; external rBFDLibraryName;

function bfd_section_size(asection: pbfd_section): Integer;

implementation

function bfd_section_size(asection: pbfd_section): Integer;
begin
  Result := asection._raw_size;
end;

end.

