(*====================================================================
    File rPeriod.pas

    Components library for Delphi, Kylix and BCB++.
    Dates period.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rPeriod.pas,v 1.2 2008/03/31 15:12:37 ariafan Exp $
======================================================================*)
unit rPeriod;

{$I POLARIS.INC}

interface

uses
  SysUtils, DateUtil, rUtils, rConst;

const
  HalfYearItems: array[1..2] of string[2] = ('I','II');
  QuartItems: array[1..4] of string[3] = ('I','II','III','IV');

type
  TrDates = array[1..2] of TrDate;

  TTypePeriod = (bdYear,bdHalfYear,bdQuart,bdMonth,bdWeek,bdDay,bdAny);

  TrPeriod = class
  private
    DMY: TrDates;
  public
    D1, D2: TDateTime;
    procedure AssignDate(dDate: TDateTime; dTypePeriod: TTypePeriod; Offset: Integer);
    procedure AssignDates(Date1,Date2: TrDate);
    procedure AssignNum(NumPeriod,Year: Integer; dTypePeriod: TTypePeriod);
    function GetName(NeedPrefix: Boolean): string;
    function GetType: TTypePeriod;
    function GetDates: TrDates;
  end;

function GetNumPeriod(dDate: TDateTime; dTypePeriod: TTypePeriod): Integer;
function GetPeriodName(D1, D2: TDateTime; NeedPrefix: Boolean): string;
function GetPeriodType(D1, D2: TDateTime): TTypePeriod;

implementation

function GetNumPeriod(dDate: TDateTime; dTypePeriod: TTypePeriod): Integer;
var
  DMY: TrDates;
begin
  with DMY[1] do begin
    DecodeDate(dDate, Y, M, D);
    case dTypePeriod of
           bdYear: Result := Y;
       bdHalfYear: Result := (M div 7) + 1;
          bdQuart: Result := ((M-1) div 3) + 1;
          bdMonth: Result := M;
           bdWeek: Result := Trunc(dDate - rDayOfWeek(dDate) - EncodeDate(Y,1,1) + rDayOfWeek(EncodeDate(Y, 1, 1))) div 7 + 1;
            bdDay: Result := Trunc(dDate - EncodeDate(Y,1,1)) + 1;
            bdAny: Result := 0;
    else Result := 0;
    end;
  end;
end;

function GetPeriodName(D1, D2: TDateTime; NeedPrefix: Boolean): string;
var
  Prefix: string;
  DMY: TrDates;
begin
  if NeedPrefix then Prefix := srPeriodPrefix+' ' else Prefix := '';
  with DMY[1] do begin
    DecodeDate(D2, Y, M, D);
    case GetPeriodType(D1, D2) of
      bdYear:     Result := Format(srPeriodYear, [Prefix, Y]);
      bdHalfYear: Result := Format(srPeriodHalfYear, [Prefix, HalfYearItems[M div 7 +1],Y]);
      bdQuart:    Result := Format(srPeriodQrt, [Prefix, QuartItems[(M-1) div 3 +1],Y]);
      bdMonth:    Result := Format(srPeriodMon, [Prefix, LongMonthNames[M],Y]);
      bdDay:      Result := Format(srPeriodDay, [Prefix, AdaptedDateStr(D1)]);
    else
      if NeedPrefix then
        Result := Format(srPeriodCustomPrfx, [DateToStr(D1), DateToStr(D2)])
      else
        Result := Format(srPeriodCustom, [DateToStr(D1), DateToStr(D2)]);
    end;
  end;
end;

function GetPeriodType(D1, D2: TDateTime): TTypePeriod;
var
  DMY: TrDates;
begin
  with DMY[1] do DecodeDate(D1, Y, M, D);
  with DMY[2] do begin
    DecodeDate(D2, Y, M, D);
    {  }
    if (rDayOfWeek(D1) = 1) and (rDayOfWeek(D2) = 7) and
       (Trunc(D2-D1) = 6) then
      Result := bdWeek
    else
    if (DMY[1].Y = Y) then
      {  }
      if (DMY[1].D = D) and (DMY[1].M = M) then
        Result := bdDay
      else
        if (DMY[1].D = 1) then
          {  }
          if (DMY[1].M = 1) and (M = 12) and (D = 31) then
            Result := bdYear
          {  }
          else if ((DMY[1].M = 1) and (D = 30) and (M = 6)) or
                  ((DMY[1].M = 7) and (D = 31) and (M = 12)) then
            Result := bdHalfYear
          {  }
          else if (DMY[1].M mod 3 = 1) and (D = DaysPerMonth(Y, M)) and
                  (M-DMY[1].M = 2) and (M mod 3 = 0) then
            Result := bdQuart
          {  }
          else if (DMY[1].M = M) and (D = DaysPerMonth(Y, M)) then
            Result := bdMonth
          else Result := bdAny
        else Result := bdAny
    else Result := bdAny;
  end;
end;

{ TrPeriod }

procedure TrPeriod.AssignDate(dDate: TDateTime; dTypePeriod: TTypePeriod; Offset: Integer);
var
  mon: Integer;
begin
  D1 := dDate;
  with DMY[1] do begin
    DecodeDate(D1, Y, M, D);
    if Offset <> 0 then begin
      mon := M;
      //    Offset
      case dTypePeriod of
            bdYear: Inc(Y, Offset);
        bdHalfYear: Y := Y + IncCycle(mon, 6 * Offset, 1, 12);
           bdQuart: Y := Y + IncCycle(mon, 3 * Offset, 1, 12);
           bdMonth: Y := Y + IncCycle(mon, Offset, 1, 12);
        bdWeek: begin
                  D1 := D1 + 7 * Offset;
                  DecodeDate(D1, Y, M, D);
                  mon := M;
                end;
        bdDay,bdAny: begin
                       D1 := D1 + Offset;
                       D2 := D1;
                       exit;
                     end;
      end;
      if D > DaysPerMonth(Y, mon) then D := DaysPerMonth(Y, mon); 
      D1 := EncodeDate(Y, mon, D);
    end;
  end;
  AssignNum(GetNumPeriod(D1, dTypePeriod), DMY[1].Y, dTypePeriod);
end;

procedure TrPeriod.AssignDates(Date1,Date2: TrDate);
begin
  with Date1 do D1 := EncodeDate(Y, M, D);
  with Date2 do D2 := EncodeDate(Y, M, D);
end;

procedure TrPeriod.AssignNum(NumPeriod, Year: Integer; dTypePeriod: TTypePeriod);
begin
  with DMY[1] do begin
    D := 1;
    Y := Year;
    DMY[2].Y := Year;
    case dTypePeriod of
      bdYear: begin
                DMY[2].D := 31;
                M := 1;
                DMY[2].M := 12;
              end;
      bdHalfYear: begin
                    DMY[2].M := NumPeriod * 6;
                    M := DMY[2].M - 5;
                    DMY[2].D := DaysPerMonth(Y, DMY[2].M);
                  end;
      bdQuart: begin
                 DMY[2].M := NumPeriod * 3;
                 M := DMY[2].M - 2;
                 DMY[2].D := DaysPerMonth(Y, DMY[2].M);
               end;
      bdMonth: begin
                 DMY[2].M := NumPeriod;
                 M := DMY[2].M;
                 DMY[2].D := DaysPerMonth(Y, DMY[2].M);
               end;
      bdWeek: begin
                D1 := EncodeDate(Y, 1, 1) - rDayOfWeek(EncodeDate(Y, 1, 1)) +
                      (NumPeriod - 1) * 7 + 1;
                D2 := D1 + 6;
                exit;
              end;
      bdDay: begin
               D1 := EncodeDate(Y, 1, 1) + NumPeriod - 1;
               D2 := D1;
               exit;
             end;
      else exit;
    end;
  end;
  AssignDates(DMY[1], DMY[2]);
end;

function TrPeriod.GetName(NeedPrefix: Boolean): string;
begin
  Result := GetPeriodName(D1, D2, NeedPrefix);
end;

function TrPeriod.GetType: TTypePeriod;
begin
  Result := GetPeriodType(D1, D2);
end;

function TrPeriod.GetDates: TrDates;
begin
  with Result[1] do DecodeDate(D1, Y, M, D);
  with Result[2] do DecodeDate(D2, Y, M, D);
end;

end.
