(*====================================================================
    File rplugset.pas

    Components library for Delphi and BCB++.
       .
    Plugins setup form.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rPlugSet.pas,v 1.3 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rPlugSet;

{$I POLARIS.INC}

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, Buttons, ComCtrls, StdCtrls,
  ExtCtrls, Consts, rConst, rinterface, {$IFDEF POLARIS_D5}rplugmoniker, {$ENDIF}rPlugins;

type
  TfmPlugsSet = class(TForm)
    box: TScrollBox;
    stat: TStatusBar;
    pnClient: TPanel;
    pnButtons: TPanel;
    btOk: TBitBtn;
    btCancel: TBitBtn;
    procedure boxResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

{$IFDEF POLARIS_D5}
function SetupPlugins(aMoniker: IrPluginMoniker): Boolean; overload;
{$ENDIF}
function SetupPlugins(aPlugins: TrPlugins): Boolean; overload;

implementation

{$R *.DFM}

procedure TfmPlugsSet.boxResize(Sender: TObject);
var
  i : Integer;
begin
  for i := 0 to box.ComponentCount - 1 do
    if (box.Components[i] is TComboBox) then
       (box.Components[i] as TControl).Width := box.Width - 20;
  btOk.Left := Width - 2 * btOk.Width - 30;
  btCancel.Left := Width - btCancel.Width - 20;
end;

{$IFDEF POLARIS_D5}
function SetupPlugins(aMoniker: IrPluginMoniker): Boolean;
  //
  procedure __Call(aResult: HRESULT; aObject: IrUnknown);
  var
    m: IrMemHolder;
    r: HRESULT;
  begin
    if not BOOL(aResult) then Exit;
    if (aObject <> nil) then begin
      r := aObject.GetLastErrorMsg(m);
      if BOOL(r)
        then raise Exception.CreateFmt(SMsgDlgError + ' %d', [r])
        else raise Exception.Create(rs(m));
    end;
  end;
  //
var
  i, j: Integer;
  lb: TLabel;
  cb: TComboBox;
  slTypes: TStringList;
  m: IrMemHolder;
begin
  slTypes := TStringList.Create;
  with TfmPlugsSet.Create(nil) do try

    __Call(aMoniker.GetClassInterfaces(nil, m), aMoniker);
    slTypes.Text := rs(m);
    for i := 0 to slTypes.Count - 1 do begin
      lb := TLabel.Create(box);
      lb.Parent := box;
      lb.Caption := slTypes.Strings[i]+':';
      lb.Left := 10;
      lb.Top := 10+40*(i);
      lb.Tag := 0;
      cb := TComboBox.Create(box);
      cb.Parent := box;
      __Call(aMoniker.GetClassNames(rs(slTypes.Strings[i]), m), aMoniker);
      cb.Items.Text := rs(m);
      cb.Left := 10;
      cb.Top := 25+40*(i);
      cb.Width := box.Width-20;
      cb.Style := csDropDownList;
      cb.Tag := i + 1;
      __Call(aMoniker.GetDefaultClassName(rs(slTypes.Strings[i]), m), aMoniker);
      j := cb.Items.IndexOf(rs(m));
      if j >= 0 then cb.ItemIndex := j;
    end;

    Caption := srPlugSetCaption;
    Result := ShowModal = mrOK;
    if Result then begin
      with box do for i := 0 to ComponentCount - 1 do
        if Components[i].Tag <> 0 then aMoniker.SetDefaultClassName(
          rs(slTypes.Strings[Components[i].Tag - 1]), rs(TComboBox(Components[i]).Text));
      aMoniker.StoreSettings;
    end;

  finally
    Free;
    slTypes.Free;
  end;
end;
{$ENDIF}

function SetupPlugins(aPlugins: TrPlugins): Boolean;
var
  i,j: Integer;
  lb: TLabel;
  cb: TComboBox;
  slTypes: TStringList;
begin
  slTypes := TStringList.Create;
  with TfmPlugsSet.Create(nil) do try

    slTypes.Text := aPlugins.GetPlugTypes;
    for i := 0 to slTypes.Count - 1 do begin
      lb := TLabel.Create(box);
      lb.Parent := box;
      lb.Caption := slTypes.Strings[i]+':';
      lb.Left := 10;
      lb.Top := 10+40*(i);
      lb.Tag := 0;
      cb := TComboBox.Create(box);
      cb.Parent := box;
      cb.Items.Text := aPlugins.GetPlugNames(slTypes.Strings[i]);
      cb.Left := 10;
      cb.Top := 25+40*(i);
      cb.Width := box.Width-20;
      cb.Style := csDropDownList;
      cb.Tag := i + 1;
      j := cb.Items.IndexOf(aPlugins.GetDefaultPlug(slTypes.Strings[i]));
      if j >= 0 then cb.ItemIndex := j;
    end;

    Caption := srPlugSetCaption;
    Result := ShowModal = mrOK;
    if Result then begin
      with box do for i := 0 to ComponentCount - 1 do
        if Components[i].Tag <> 0 then aPlugins.SetDefaultPlug(
          slTypes.Strings[Components[i].Tag - 1], TComboBox(Components[i]).Text);
      aPlugins.SaveParamsToReg;
    end;

  finally
    Free;
    slTypes.Free;
  end;
end;

end.

