(*====================================================================
    File rRegIni.pas

    Components library for Delphi and BCB++.
    Registry utils

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rRegIni.pas,v 1.4 2008/10/06 11:58:10 khmelevskiy Exp $
======================================================================*)
unit rRegIni;

{$I POLARIS.INC}

interface

uses
  {$IFDEF LINUX}Types,{$endif}
  {$ifdef mswindows} Windows, Registry, {$endif}
  Classes, IniFiles, SysUtils, {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils{$ENDIF}, Math, rConst;

{$IFDEF POLARIS_D6}
var GlobalConfigFile: string = '';
{$ELSE}
const GlobalConfigFile: string = '';
{$ENDIF}

{$ifndef mswindows}
type
  HKEY = type LongWord;
const
  {$EXTERNALSYM HKEY_CLASSES_ROOT}
  HKEY_CLASSES_ROOT     = DWORD($80000000);
  {$EXTERNALSYM HKEY_CURRENT_USER}
  HKEY_CURRENT_USER     = DWORD($80000001);
  {$EXTERNALSYM HKEY_LOCAL_MACHINE}
  HKEY_LOCAL_MACHINE    = DWORD($80000002);
  {$EXTERNALSYM HKEY_USERS}
  HKEY_USERS            = DWORD($80000003);
  {$EXTERNALSYM HKEY_PERFORMANCE_DATA}
  HKEY_PERFORMANCE_DATA = DWORD($80000004);
  {$EXTERNALSYM HKEY_CURRENT_CONFIG}
  HKEY_CURRENT_CONFIG   = DWORD($80000005);
  {$EXTERNALSYM HKEY_DYN_DATA}
  HKEY_DYN_DATA         = DWORD($80000006);
{$endif}

type
  TrRegIniMode = (rModeRegistry, rModeIni);

  TrRegIni = class(TComponent)
  private
    FCurrentPath: string;
    {$ifdef mswindows}
    FRegistry: TRegistry;
    {$endif}
    FRegIniMode: TrRegIniMode;
    FHKey: HKEY;
    FIniFile: TIniFile;
    FIniFileName: TFileName;
    FIniFileFullName: TFileName;
    procedure SetIniFileName(const aValue: TFileName);
    procedure SetRootKey(Value: HKEY);
    function GetCurrentKey: HKey;
    // make path absolute
    function CheckDef(const Name: string): String;
    //       16- 
    function BufToStr(var Buffer; BufSize: Integer): string;
    // Return TRUE, if Key is SubKey of ParentKey
    procedure SetRegIniMode(const aValue: TrRegIniMode);
    procedure RecalcIniFileName();
  public
    constructor Create(AOwner: TComponent); override;
    constructor CreateIni(AOwner: TComponent; const aFileName: TFileName);
    destructor Destroy; override;
    {$ifdef mswindows}
    property Registry: TRegistry read FRegistry;
    {$endif}
    property IniFile: TIniFile read FIniFile;
    property CurrentKey: HKey read GetCurrentKey;
    property CurrentPath: string read FCurrentPath;
    property RegIniMode: TrRegIniMode read FRegIniMode write SetRegIniMode;
    // some procedures imported from TRegistry
    procedure CloseKey;
    function CreateKey(const Key: string): boolean;
    function DeleteKey(const Key: string): Boolean;
    function DeleteValue(const Name: string): Boolean;
    procedure GetKeyNames(Strings: TStrings);
    procedure GetValueNames(Strings: TStrings);
    function HasSubKeys: Boolean;
    function KeyExists(const Key: string): Boolean;
    function OpenKey(const Key: string; CanCreate: Boolean): Boolean;
    function ReadCurrency(const Name: string): Currency;
    function ReadBinaryData(const Name: string; var Buffer; BufSize: Integer): Integer;
    function ReadBool(const Name: string): Boolean;
    function ReadDate(const Name: string): TDateTime;
    function ReadDateTime(const Name: string): TDateTime;
    function ReadFloat(const Name: string): Double;
    function ReadInteger(const Name: string): Integer;
    function ReadString(const Name: string): string;
    function ReadTime(const Name: string): TDateTime;
    function ValueExists(const Name: string): Boolean;
    procedure WriteCurrency(const Name: string; Value: Currency);
    procedure WriteBinaryData(const Name: string; var Buffer; BufSize: Integer);
    procedure WriteBool(const Name: string; Value: Boolean);
    procedure WriteDate(const Name: string; Value: TDateTime);
    procedure WriteDateTime(const Name: string; Value: TDateTime);
    procedure WriteFloat(const Name: string; Value: Double);
    procedure WriteInteger(const Name: string; Value: Integer);
    procedure WriteString(const Name, Value: string);
    procedure WriteTime(const Name: string; Value: TDateTime);
    // ---------------------------------------
  published
    property RootKey: HKEY read FHKey write SetRootKey;
    property IniFileName: TFileName read FIniFileName write SetIniFileName;
  end;

implementation

uses rUtils;

{ TrRegIni }

constructor TrRegIni.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHKey := HKEY_CURRENT_USER;
  IniFileName := GlobalConfigFile;
  {$ifdef mswindows}
  FRegIniMode := rModeIni;
  SetRegIniMode(rModeRegistry);
  {$else}
  FRegIniMode := rModeRegistry;
  RecalcIniFileName();
  SetRegIniMode(rModeIni);
  {$endif}
end;

procedure TrRegIni.RecalcIniFileName();
var
  s: string;
begin
  if ExtractFilePath(FIniFileName) <> '' then begin
    FIniFileFullName := FIniFileName;
    FIniFileName := ExtractFileName(FIniFileName);
  end
  else begin
    if FIniFileName = '' then
      FIniFileName := ChangeFileExt(ExtractFileName(GetModulePath(0)), '.regini');
    if FHKey = HKEY_LOCAL_MACHINE
      then s := GetGlobalConfigDir()
      else s := GetLocalConfigDir();
    FIniFileFullName := AddSlash(s) + FIniFileName;
  end;
  if (FRegIniMode = rModeIni) and (FIniFile.FileName <> FIniFileFullName) then begin
    if FileExists(FIniFile.FileName) or DirectoryExists(ExtractFilePath(FIniFile.FileName)) then FIniFile.UpdateFile;
    FIniFile.Free;
    FIniFile := TIniFile.Create(FIniFileFullName);
    {$ifdef linux}
    FIniFile.CaseSensitive := False;
    {$endif}
  end;
end;

procedure TrRegIni.SetRootKey(Value: HKEY);
begin
  FHKey := Value;
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then FRegistry.RootKey := Value
  else
  {$endif}
  RecalcIniFileName();
end;

procedure TrRegIni.SetIniFileName(const aValue: TFileName);
begin
  if FIniFileName = aValue then Exit;
  FIniFileName := aValue;
  if FIniFileName = '' then begin
    FIniFileFullName := '';
    Exit;
  end
  else RecalcIniFileName();
end;

constructor TrRegIni.CreateIni(AOwner: TComponent; const aFileName: TFileName);
begin
  inherited Create(AOwner);
  FRegIniMode := rModeRegistry;
  SetRegIniMode(rModeIni);
  IniFileName := aFileName;
end;

function TrRegIni.GetCurrentKey: HKey;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then Result := FRegistry.CurrentKey
  else
  {$endif}
  Result := FHKey;
end;

procedure TrRegIni.CloseKey;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then FRegistry.CloseKey
  else
  {$endif}
    if FileExists(FIniFile.FileName) or DirectoryExists(ExtractFilePath(FIniFile.FileName)) then FIniFile.UpdateFile;
  FCurrentPath := '';
end;

function TrRegIni.CreateKey(const Key: string): Boolean;
begin
  Result := False;
  if Key = '' then Exit;
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then begin
    Result := FRegistry.CreateKey(Key);
    FCurrentPath := FRegistry.CurrentPath;
  end
  else
  {$endif}
  begin
    if Key[1] <> '\' then begin
      if FCurrentPath <> '' then FCurrentPath := FCurrentPath + '\';
      FCurrentPath := FCurrentPath + Key;
    end
    else FCurrentPath := Copy(Key, 2, MaxInt);
    Result := True;
  end;
end;

function TrRegIni.DeleteKey(const Key: string): Boolean;
var
  s: string;
  sl: TStringList;
  i: Integer;
begin
  Result := False;
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then begin
    Result := FRegistry.DeleteKey(Key);
    FCurrentPath := FRegistry.CurrentPath;
  end
  else
  {$endif}
  begin
    if Key = '' then Exit;
    s := Key;
    if s[1] = '\'
      then s := Copy(s, 2, MaxInt)
      else s := FCurrentPath + '\' + s;
    sl := TStringList.Create();
    try
      GetKeyNames(sl);
      for i := 0 to sl.Count - 1 do
        DeleteKey('\' + s + '\' + sl[i]);
    finally
      sl.Free();
    end;
    FIniFile.EraseSection(s);
    Result := True;
  end;
end;

function TrRegIni.DeleteValue(const Name: string): Boolean;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.DeleteValue(Name)
  else
  {$endif}
  begin
    Result := (FCurrentPath <> '') and FIniFile.SectionExists(FCurrentPath);
    if not Result then Exit;
    FIniFile.DeleteKey(FCurrentPath, Name);
  end;
end;

procedure TrRegIni.GetKeyNames(Strings: TStrings);
var
  i: Integer;
  s: string;
  sl1, sl2: TStringList;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.GetKeyNames(Strings)
  else
  {$endif}
  begin
    sl1 := TStringList.Create();
    sl2 := TStringList.Create();
    try
      sl2.Sorted := True;
      sl2.Duplicates := dupIgnore;
      FIniFile.ReadSections(sl1);
      for i := 0 to sl1.Count - 1 do begin
        if Copy(sl1[i], 1, Length(FCurrentPath)) = FCurrentPath
        then begin
          s := Copy(sl1[i], Length(FCurrentPath) + 2, MaxInt);
          if s <> '' then sl2.Add(ExtractWord(1, s, ['\']))
        end
      end;
      Strings.Assign(sl2);
    finally
      sl1.Free();
      sl2.Free();
    end;
  end;
end;

procedure TrRegIni.GetValueNames(Strings: TStrings);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.GetValueNames(Strings)
  else
  {$endif}
    FIniFile.ReadSection(FCurrentPath, Strings);
end;

function TrRegIni.HasSubKeys: Boolean;
var
  sl: TStringList;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.HasSubKeys()
  else
  {$endif}
  begin
    sl := TStringList.Create();
    try
      GetKeyNames(sl);
      Result := sl.Count > 0;
    finally
      sl.Free();
    end;
  end;
end;

function TrRegIni.KeyExists(const Key: string): Boolean;
var
  s: string;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.KeyExists(Key)
  else
  {$endif}
  begin
    if Key = '' then begin
      Result := False;
      Exit;
    end;
    if Key[1] <> '\' then begin
      if FCurrentPath <> ''
        then s := FCurrentPath + '\' + Key
        else s := Key;
    end
    else s := Copy(Key, 2, MaxInt);
    Result := FIniFile.SectionExists(s);
  end;
end;

function TrRegIni.OpenKey(const Key: string; CanCreate: Boolean): Boolean;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.OpenKey(Key, CanCreate)
  else
  {$endif}
  begin
    if CanCreate then Result := CreateKey(Key)
    else begin
      Result := KeyExists(Key);
      if Result then begin
        if Key[1] <> '\' then begin
          if FCurrentPath <> '' then FCurrentPath := FCurrentPath + '\';
          FCurrentPath := FCurrentPath + Key;
        end
        else FCurrentPath := Copy(Key, 2, MaxInt);
      end;
    end;
  end;
end;

function TrRegIni.ReadBinaryData(const Name: string; var Buffer; BufSize: Integer): Integer;
var
  s: string;
  i: integer;
  p: pByte;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadBinaryData(Name, Buffer, BufSize)
  else
  {$endif}
  begin
    s := FIniFile.ReadString(FCurrentPath, Name, '');
    Result := Min(Length(s) div 2, BufSize);
    p := @Buffer;
    for i := 0 to Result - 1 do begin
      p^ := Hex2Dec(Copy(S, 2*i + 1, 2));
      Inc(p);
    end;
  end;
end;

function TrRegIni.ReadBool(const Name: string): Boolean;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadBool(Name)
  else
  {$endif}
    Result := FIniFile.ReadBool(FCurrentPath, CheckDef(Name), False);
end;

function TrRegIni.ReadCurrency(const Name: string): Currency;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadCurrency(Name)
  else
  {$endif}
    Result := FIniFile.ReadFloat(FCurrentPath, CheckDef(Name), 0);
end;

function TrRegIni.ReadDate(const Name: string): TDateTime;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadDate(Name)
  else
  {$endif}
    Result := FIniFile.ReadDate(FCurrentPath, CheckDef(Name), 0);
end;

function TrRegIni.ReadDateTime(const Name: string): TDateTime;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadDateTime(Name)
  else
  {$endif}
    Result := FIniFile.ReadDateTime(FCurrentPath, CheckDef(Name), 0);
end;

function TrRegIni.ReadFloat(const Name: string): Double;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadFloat(Name)
  else
  {$endif}
    Result := FIniFile.ReadFloat(FCurrentPath, CheckDef(Name), 0);
end;

function TrRegIni.ReadInteger(const Name: string): Integer;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadInteger(Name)
  else
  {$endif}
    Result := FIniFile.ReadInteger(FCurrentPath, CheckDef(Name), 0);
end;

function TrRegIni.ReadString(const Name: string): string;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadString(Name)
  else
  {$endif}
    Result := FIniFile.ReadString(FCurrentPath, CheckDef(Name), '');
end;

function TrRegIni.ReadTime(const Name: string): TDateTime;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ReadTime(Name)
  else
  {$endif}
    Result := FIniFile.ReadTime(FCurrentPath, CheckDef(Name), 0);
end;

function TrRegIni.ValueExists(const Name: string): Boolean;
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    Result := FRegistry.ValueExists(Name)
  else
  {$endif}
    Result := FIniFile.ValueExists(FCurrentPath, Name);
end;

procedure TrRegIni.WriteBinaryData(const Name: string; var Buffer; BufSize: Integer);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteBinaryData(Name, Buffer, BufSize)
  else
  {$endif}
    FIniFile.WriteString(FCurrentPath, Name, BufToStr(Buffer, BufSize));
end;

procedure TrRegIni.WriteBool(const Name: string; Value: Boolean);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteBool(Name, Value)
  else
  {$endif}
    FIniFile.WriteBool(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteCurrency(const Name: string; Value: Currency);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteCurrency(Name, Value)
  else
  {$endif}
    FIniFile.WriteFloat(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteDate(const Name: string; Value: TDateTime);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteDate(Name, Value)
  else
  {$endif}
    FIniFile.WriteDate(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteDateTime(const Name: string; Value: TDateTime);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteDateTime(Name, Value)
  else
  {$endif}
    FIniFile.WriteDateTime(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteFloat(const Name: string; Value: Double);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteFloat(Name, Value)
  else
  {$endif}
    FIniFile.WriteFloat(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteInteger(const Name: string; Value: Integer);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteInteger(Name, Value)
  else
  {$endif}
    FIniFile.WriteInteger(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteString(const Name, Value: string);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteString(Name, Value)
  else
  {$endif}
    FIniFile.WriteString(FCurrentPath, CheckDef(Name), Value);
end;

procedure TrRegIni.WriteTime(const Name: string; Value: TDateTime);
begin
  {$ifdef mswindows}
  if FRegIniMode = rModeRegistry then
    FRegistry.WriteTime(Name, Value)
  else
  {$endif}
    FIniFile.WriteTime(FCurrentPath, CheckDef(Name), Value);
end;

function TrRegIni.CheckDef(const Name: string): String;
begin
  if Name = '' then Result := 'DEFAULT' else Result := Name;
end;

function TrRegIni.BufToStr(var Buffer; BufSize: Integer): string;
var
  i: integer;
  s: string[2];
  p: pByte;
begin
  SetLength(Result, 2 * BufSize);
  p := @Buffer;
  for i := 0 to BufSize - 2 do begin
    s := Dec2Hex(p^, 2);
    Result[2*i] := {$IFDEF POLARIS_D12}Char({$ENDIF}s[1]{$IFDEF POLARIS_D12}){$ENDIF};
    Result[2*i + 1] := {$IFDEF POLARIS_D12}Char({$ENDIF}s[2]{$IFDEF POLARIS_D12}){$ENDIF};
    Inc(p);
  end;
end;

procedure TrRegIni.SetRegIniMode(const aValue: TrRegIniMode);
begin
  if FRegIniMode <> aValue then begin
    {$ifndef mswindows}
    if aValue = rModeRegistry then
      raise Exception.Create(srNoRegistrySupported);
    {$endif}
    case aValue of
      rModeRegistry: begin
        FIniFile.Free;
        FIniFile := nil;
        {$ifdef mswindows}
        FRegistry := TRegistry.Create();
        {$endif}
      end;
      rModeIni: begin
        {$ifdef mswindows}
        FRegistry.Free;
        FRegistry := nil;
        {$endif}
        FIniFile := TIniFile.Create(FIniFileFullName);
        {$ifdef linux}
        FIniFile.CaseSensitive := False;
        {$endif}
      end;
    end;
    FRegIniMode := aValue;
  end;
end;

destructor TrRegIni.Destroy;
begin
  if FIniFile <> nil then begin
    if FileExists(FIniFile.FileName) or DirectoryExists(ExtractFilePath(FIniFile.FileName)) then FIniFile.UpdateFile;
    FIniFile.Free;
  end;
  {$ifdef mswindows}
  FRegistry.Free;
  {$endif}
  inherited;
end;

end.

