(*====================================================================
    File rinterface.pas

    Components library for Delphi and BCB++.
    
    Interfaces

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rinterface.pas,v 1.4 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit rinterface;

{$I POLARIS.INC}

interface

uses
{$ifdef mswindows} Windows {$endif}
{$IFDEF LINUX} Types {$ENDIF};

const
  IID_Unknown = '{00000000-0000-0000-C000-000000000046}';
  IID_rMemHolder = '{785CCDEE-D301-4BDE-8322-DD0BFC38E794}';
type
  IrMemHolder = interface
  [IID_rMemHolder]
    function Alloc(aSize: Integer): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function FillAlloc(aSize: Integer; aChar: Char): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function AllocFromString(aStr: PChar): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function AllocFromMem(aMem: Pointer; aSize: Integer): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetPtr(out aPtr: PChar): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetSize(out aSize: Integer): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Clear(): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

// error codes
const
  PE_rMemHolder_Base            = 0;
  PE_rMemHolder_Alloc           = PE_rMemHolder_Base + 1;
  PE_rMemHolder_FillAlloc       = PE_rMemHolder_Base + 2;
  PE_rMemHolder_AllocFromString = PE_rMemHolder_Base + 3;
  PE_rMemHolder_AllocFromMem    = PE_rMemHolder_Base + 4;
  PE_rMemHolder_GetPtr          = PE_rMemHolder_Base + 5;
  PE_rMemHolder_GetSize         = PE_rMemHolder_Base + 6;
  PE_rMemHolder_Clear           = PE_rMemHolder_Base + 7;

const
  IID_rUnknown = '{ECF960C4-D35F-4304-BD01-0E21EB3B523A}';
  IID_rCustomMoniker = '{9A43CA10-508F-495E-81B9-25E891CBAE51}';
type
  IrUnknown = interface(IUnknown)
  [IID_rUnknown]
    function _SetOwnerInterface(aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastError: HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOwnerInterface(out aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassDisplayName(out aName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassModuleName(out aName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetHaveRefCount(out aHave: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Dispose(aMode: DWORD): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

const
  PE_rUnknown_Base                    = 50;
  PE_rUnknown_SetOwnerInterface       = PE_rUnknown_Base + 1;
  PE_rUnknown_GetLastError            = PE_rUnknown_Base + 2;
  PE_rUnknown_GetLastErrorMsg         = PE_rUnknown_Base + 3;
  PE_rUnknown_GetOwnerInterface       = PE_rUnknown_Base + 4;
  PE_rUnknown_GetClassDisplayName     = PE_rUnknown_Base + 5;
  PE_rUnknown_GetInterfaceCount       = PE_rUnknown_Base + 6;
  PE_rUnknown_GetInterfaceID          = PE_rUnknown_Base + 7;
  PE_rUnknown_GetInterfaceDisplayName = PE_rUnknown_Base + 8;
  PE_rUnknown_GetClassModuleName      = PE_rUnknown_Base + 9;
  PE_rUnknown_GetHaveRefCount         = PE_rUnknown_Base + 10;
  PE_rUnknown_Dispose                 = PE_rUnknown_Base + 11;

  PE_rUnknown_CapabilityNotSupported  = PE_rUnknown_Base + 20;
  PE_rUnknown_UnknownInterface        = PE_rUnknown_Base + 21;

type
  IrCustomMoniker = interface(IrUnknown)
  [IID_rCustomMoniker]
    function _DecInstanceCounter(aObject: IrUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function NewObjectInstance(aClassName, aInterfaceName: IrMemHolder;
      aDefaultIfNotExists: Boolean; out aObject: IrUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceByDisplayName(aName: IrMemHolder; out aID: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassesCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassInterfaces(aClassName: IrMemHolder; out aInterfaceNames: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassNames(aInterfaceName: IrMemHolder; out aClassNames: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetDefaultClassName(aInterfaceName: IrMemHolder; out aClassName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function SetDefaultClassName(aInterfaceName, aClassName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetActiveObjectsCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

const
  PE_rCustomMoniker_Base                      = 100;
  PE_rCustomMoniker_DecInstanceCounter        = PE_rCustomMoniker_Base + 1;
  PE_rCustomMoniker_NewObjectInstance         = PE_rCustomMoniker_Base + 2;
  PE_rCustomMoniker_GetInterfaceByDisplayName = PE_rCustomMoniker_Base + 3;
  PE_rCustomMoniker_GetClassesCount           = PE_rCustomMoniker_Base + 4;
  PE_rCustomMoniker_GetClassInterfaces        = PE_rCustomMoniker_Base + 5;
  PE_rCustomMoniker_GetClassNames             = PE_rCustomMoniker_Base + 6;
  PE_rCustomMoniker_GetDefaultClassName       = PE_rCustomMoniker_Base + 7;
  PE_rCustomMoniker_SetDefaultClassName       = PE_rCustomMoniker_Base + 8;
  PE_rCustomMoniker_GetActiveObjectsCount     = PE_rCustomMoniker_Base + 9;

const
  IID_rCustomPlugin = '{DF215D35-F0B8-469A-8DB0-4D74B22B11F9}';
type
  IrCustomPlugin = interface(IrUnknown)
  [IID_rCustomPlugin]
    function GetOptParams(out aParams: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function SetCallback(aCallback: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetCallback(out aCallback: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

const
  IID_rCustomPlugin2 = '{BAF455E7-3640-40D9-A49D-F18EA4C192DE}';
type
  IrCustomPlugin2 = interface(IrCustomPlugin)
  [IID_rCustomPlugin2]
    function CheckOptParam(aParamName, aParamValue: IrMemHolder): Boolean;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

const
  PE_rCustomPlugin_Base               = 150;
  PE_rCustomPlugin_GetOptParams       = PE_rCustomPlugin_Base + 1;
  PE_rCustomPlugin_SetCallback        = PE_rCustomPlugin_Base + 2;
  PE_rCustomPlugin_GetCallback        = PE_rCustomPlugin_Base + 3;
  PE_rCustomPlugin_CheckOptParam      = PE_rCustomPlugin_Base + 4;


const
  IID_rPluginMoniker = '{9BAF397F-59C8-44EC-BEA6-A41C0E64CB06}';
type
  IrPluginMoniker = interface(IrCustomMoniker)
  [IID_rPluginMoniker]
    function GetPlugsPath(out aPath: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function SetPlugsPath(aPath: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function RestoreSettings(): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function StoreSettings(): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetPlugByOptParam(aPlugType, aParamName, aParamValue: IrMemHolder;
      out aPlugName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOptParam(aPlugName, aParamName: IrMemHolder; out aValue: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

const
  PE_rPluginMoniker_Base              = 200;
  PE_rPluginMoniker_GetPlugsPath      = PE_rPluginMoniker_Base + 1;
  PE_rPluginMoniker_SetPlugsPath      = PE_rPluginMoniker_Base + 2;
  PE_rPluginMoniker_RestoreSettings   = PE_rPluginMoniker_Base + 3;
  PE_rPluginMoniker_StoreSettings     = PE_rPluginMoniker_Base + 4;
  PE_rPluginMoniker_GetPlugByOptParam = PE_rPluginMoniker_Base + 5;
  PE_rPluginMoniker_GetOptParam       = PE_rPluginMoniker_Base + 6;

  PE_rPluginMoniker_UnknownPlugin     = PE_rPluginMoniker_Base + 20;
  PE_rPluginMoniker_BadParams         = PE_rPluginMoniker_Base + 21;


const
  IID_rSettingsWrapper = '{C1E42BE4-0609-4DC1-A120-EB9939E349FA}';
type
  IrSettingsWrapper = interface
  [IID_rSettingsWrapper]
    procedure LoadFromFile(const aFileName: string);
    function GetText(): string;
    procedure SetText(const aValue: string);
    function OpenKey(const aKey: string; aCanCreate: Boolean = False; aIndex: Integer = 0): Boolean;
    procedure CloseKey();
    procedure SetXMLVersionEnabled(aEnabled: Boolean);
    function GetKeyXML: string;
    procedure SetKeyXML(const aValue: string);
    function GetKeyCount(const aName: string = ''): Integer;
    function GetKeyName(aIndex: Integer): string;
    function GetValueCount(): Integer;
    function GetValueName(aIndex: Integer): string;
    function KeyExists(const aName: string): Boolean;
    function ValueExists(const aName: string): Boolean;
    function HasSubKeys: Boolean;
    function GetKeyValue(const aName: string): string;
    procedure SetKeyValue(const aName, aValue: string);
    function CreateKey(const aKey: string): Boolean;
    function DeleteKey(const aKey: string): Boolean;
    function GetCurrentPath(): string;
    procedure Merge(const aText: string);
    //
    function ReadBool(const Name: string): Boolean;
    function ReadDate(const Name: string): TDateTime;
    function ReadDateTime(const Name: string): TDateTime;
    function ReadFloat(const Name: string): Double;
    function ReadInteger(const Name: string): Integer;
    function ReadString(const Name: string): string;
    function ReadTime(const Name: string): TDateTime;
    procedure WriteBool(const Name: string; Value: Boolean);
    procedure WriteDate(const Name: string; Value: TDateTime);
    procedure WriteDateTime(const Name: string; Value: TDateTime);
    procedure WriteFloat(const Name: string; Value: Double);
    procedure WriteInteger(const Name: string; Value: Integer);
    procedure WriteString(const Name, Value: string);
    procedure WriteTime(const Name: string; Value: TDateTime);
    //
    property Text: string read GetText write SetText;
    property KeyXML: string read GetKeyXML write SetKeyXML;
    property KeyCount[const aName: string]: Integer read GetKeyCount;
    property KeyNames[aIndex: Integer]: string read GetKeyName;
    property ValueCount: Integer read GetValueCount;
    property ValueNames[aIndex: Integer]: string read GetValueName;
    property Values[const aName: string]: string read ReadString write WriteString;
    property KeyValues[const aName: string]: string read GetKeyValue write SetKeyValue;
    property CurrentPath: string read GetCurrentPath;
    property VersionEnabled: Boolean write SetXMLVersionEnabled;
  end;

const
  PE_rSettingsWrapper_Base = 250;

const
  IID_rDataServer = '{553FA4C2-32A8-4B04-8FF3-F94A06BC9B63}';
type
  IrDataServer = interface(IrUnknown)
  [IID_rDataServer]
    function Open(aDataSetName: IrMemHolder; out aFields: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function IsBOF(out aBOF: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function IsEOF(out aEOF: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetRecordCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetRecord(out aRecord: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetBlobData(aFieldNumber: Integer; out aData: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Next(out aEOF: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Close(): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

const
  PE_rDataServer_Base           = 300;
  PE_rDataServer_Open           = PE_rDataServer_Base + 1;
  PE_rDataServer_IsBOF          = PE_rDataServer_Base + 2;
  PE_rDataServer_IsEOF          = PE_rDataServer_Base + 3;
  PE_rDataServer_GetRecordCount = PE_rDataServer_Base + 4;
  PE_rDataServer_GetRecord      = PE_rDataServer_Base + 5;
  PE_rDataServer_GetBlobData    = PE_rDataServer_Base + 6;
  PE_rDataServer_Next           = PE_rDataServer_Base + 7;
  PE_rDataServer_Close          = PE_rDataServer_Base + 8;


function CreateMemHolder(out aHolder: IrMemHolder): HRESULT;
  {$ifdef mswindows} stdcall external 'psmemhldr.dll'; {$endif}
  {$ifdef linux} cdecl external 'libpsmemhldr.so'; {$endif}

// Function for compatibility - do not use in the future
function NewMemHolder: IrMemHolder;
  {$ifdef mswindows} stdcall external 'psmemhldr.dll'; {$endif}
  {$ifdef linux} cdecl external 'libpsmemhldr.so'; {$endif}

function rDispose(var aIntf: IrUnknown): HRESULT;

implementation

function rDispose(var aIntf: IrUnknown): HRESULT;
var
  h: Boolean;
begin
  if aIntf = nil then begin
    Result := S_OK;
    Exit;
  end;
  Result := aIntf.GetHaveRefCount(h);
  if (Result = S_OK) then begin
    if not h then begin
      Result := aIntf.Dispose(0);
      DWORD(aIntf) := 0;
    end
    else aIntf := nil;
  end;
end;

end.

