(*====================================================================
    File rplugmoniker.pas

    Components library for Delphi, Kylix and BCB++.
    Plugins support.

    Copyright  2004-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rplugmoniker.pas,v 1.8 2008/04/15 17:29:25 ariafan Exp $
======================================================================*)
unit rplugmoniker;

{$I POLARIS.INC}

interface

uses
  {$ifdef mswindows} Windows, Registry, {$endif}
  {$ifdef linux} Types, Libc, {$endif}
  SysUtils, Classes,
  {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils, ComObj, activex{$ENDIF},
  rinterface, rUtils, rConst, rClasses, rDLLloader;

type
  TrPluginLoadError = procedure(Sender: TObject; PluginLibrary, ErrorMessage: string) of object;

  ErPlugBaseError = class(Exception);

  TrPluginLoader = class(TrDLLloader)
  private
    FCritic: TRTLCriticalSection;
    FPlugName: string;
    FSupportedTypes: TStringList;
    FOptParams: TStringList;
    FInstCount: Integer;
    FAutoUnload: Boolean;
    function CheckRequiredEntry: Boolean;
    procedure UpdateInfo;
    function GetOptParam(aParamName: string): string;
    procedure IncInstCount;
    procedure DecInstCount;
  public
    constructor Create(aOwner: TComponent); override;
    function CheckOptParam(const aParamName, aParamValue: string): Boolean;
    destructor Destroy; override;
    //
    property PlugName: string read FPlugName;
    property SupportedTypes: TStringList read FSupportedTypes;
    property OptParam[aParamName: string]: string read GetOptParam;
    property InstCount: Integer read FInstCount;
    property AutoUnload: Boolean read FAutoUnload write FAutoUnload;
  end;

  TrPluginMoniker = class(TComponent, IrUnknown, IrCustomMoniker, IrPluginMoniker)
  private
    FPlugins: TrTree;
    FRegKey: string;
    FPlugsPath: string;
    FFileMask: string;
    FRequiredVersion: string;
    FInvisibleTypes: TStringList;
    FPluginCount: Integer;
    {$ifdef mswindows}
    FReg: TRegistry;
    {$endif}
    FModifyed: Boolean;
    FCallback: IUnknown;
    FAutoUnload: Boolean;
    FOnPluginLoadError: TrPluginLoadError;
{$IFDEF POLARIS_D5}
    procedure TreeNotify(Sender: TObject; Ptr: Pointer; Action: TListNotification);
{$ENDIF}
    procedure _SetPlugsPath(const Value: string);
    function CheckVersionFormat: Boolean;
    function GetCount: Integer;
    procedure SetRequiredVersion(const Value: string);
    function IsKnownIID(aIID: TGUID; out aTypeDescript: Pointer): Boolean;
    procedure RegisterType(aIID: TGUID; const aTypeName: string; aPlugin: TrPluginLoader; out aTypeDescript: Pointer);
    function GetTypeByTypeName(aPlugType: string; var aTypeStruct: Pointer): TrList;
  protected
    FLastError: Integer;
    FLastErrorMsg: string;
    function SetError(aCode: HRESULT = S_OK; aExc: Exception = nil; aMsg: string = crEmptyStr): HRESULT;
    function SelectPlugin(const aPlugType, aPlugName: string; aDefaultIfNotExists: Boolean): TrPluginLoader;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    function GetPlugTypes(const aPlugName: string = crEmptyStr): string;
    function GetPlugNames(const aPlugType: string = crEmptyStr): string;
    function GetDefaultPlug(const aPlugType: string): string;
    function GetPlugByOptParams(aPlugType, aParamName, aParamValue: string): string;
    function GetPluginByName(const aPlugName: string): TrPluginLoader;
    function GetDefaultPlugin(const aPlugType: string): TrPluginLoader;
    function PlugExists(const aPlugType, aPlugName: string): Boolean;
    procedure SetDefaultPlug(const aPlugType, aPlugName: string);
    function GetPlugTypeByGUID(aGUID: TGUID): string;

    // == IrUnknown ==
    function _SetOwnerInterface(aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastError: HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOwnerInterface(out aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassDisplayName(out aName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassModuleName(out aName: IrMemHolder): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetHaveRefCount(out aHave: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Dispose(aMode: DWORD): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    // == IrCustomMoniker ==
    function _DecInstanceCounter(aObject: IrUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function NewObjectInstance(aClassName, aInterfaceName: IrMemHolder;
      aDefaultIfNotExists: Boolean; out aObject: IrUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceByDisplayName(aName: IrMemHolder; out aID: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassesCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassInterfaces(aClassName: IrMemHolder; out aInterfaceNames: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassNames(aInterfaceName: IrMemHolder; out aClassNames: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetDefaultClassName(aInterfaceName: IrMemHolder; out aClassName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function SetDefaultClassName(aInterfaceName, aClassName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetActiveObjectsCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    // == IrPluginMoniker ==
    function GetPlugsPath(out aPath: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function SetPlugsPath(aPath: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function RestoreSettings(): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function StoreSettings(): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetPlugByOptParam(aPlugType, aParamName, aParamValue: IrMemHolder;
      out aPlugName: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOptParam(aPlugName, aParamName: IrMemHolder; out aValue: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    property PlugsPath: string read FPlugsPath write _SetPlugsPath;
    property Count: Integer read GetCount;
    property Callback: IUnknown read FCallback write FCallback;
  published
    property RegKey: string read FRegKey write FRegKey;
    property FileMask: string read FFileMask write FFileMask stored True;
    property RequiredVersion: string read FRequiredVersion write SetRequiredVersion stored CheckVersionFormat;
    property AutoUnload: Boolean read FAutoUnload write FAutoUnload default True;
    property OnPluginLoadError: TrPluginLoadError read FOnPluginLoadError write FOnPluginLoadError;
  end;

  TrCustomPlugin = class(TObject, IUnknown, IrUnknown, IrCustomPlugin)
  private
    FModuleName: string;
    FClassDisplayName: string;
    FCallback: IUnknown;
    function __SafeFree(): HRESULT;
  protected
    FMoniker: IrPluginMoniker;
    FLastError: HRESULT;
    FLastErrorMsg: string;
{$ifndef ps_nopluginrefcount}
    FRefCount: Integer;
  {$ifdef mswindows}
    FReleaseData: array[0..2] of Pointer;
  {$endif}
{$endif}

    function SetError(aErrCode: HRESULT = S_OK; aExc: Exception = nil; aMsg: string = crEmptyStr): HRESULT; virtual;
    function QueryPlugin(aInterfaceName, aClassName: string;
      aDefaultIfNotExists, aRaiseIfNotFound: Boolean; out aObject): HRESULT;
    function CallPlugin(aResult: HRESULT; aPlugin: IrUnknown): HRESULT;
    function QueryCallback(aIID: TGUID; out aIntf): HRESULT;

    // == IUnknown ==
    function  QueryInterface(const IID: TGUID; out Obj): HRESULT; virtual; stdcall;
    function _AddRef: Integer; virtual; stdcall;
    function _Release: Integer; virtual; stdcall;
  public
    constructor Create; virtual;

    // == IrUnknown ==
    function _SetOwnerInterface(aOwner: IUnknown): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastError: HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOwnerInterface(out aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassDisplayName(out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF} abstract;
    function GetInterfaceCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF} abstract;
    function GetClassModuleName(out aName: IrMemHolder): HRESULT;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetHaveRefCount(out aHave: Boolean): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Dispose(aMode: DWORD): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    // == IrCustomPlugin ==
    function GetOptParams(out aParams: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF} abstract;
    function SetCallback(aCallback: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetCallback(out aCallback: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    //
    property LastErrorMsg: string read FLastErrorMsg;
    property ModuleName: string read FModuleName;
  end;

  TrPluginThread = class(TThread, IUnknown, IrUnknown)
  protected
    FLastError: HRESULT;
    FLastErrorMsg: string;
    FMoniker: IrPluginMoniker;
    function SetError(aCode: HRESULT = S_OK; aExc: Exception = nil; aMsg: string = crEmptyStr): HRESULT;
    function QueryPlugin(aInterfaceName, aClassName: string;
      aDefaultIfNotExists, aRaiseIfNotFound: Boolean; out aObject: IrUnknown): HRESULT;
    function CallPlugin(aResult: HRESULT; aPlugin: IrCustomPlugin): HRESULT;
  public

    // === IUnknown ===
    function QueryInterface(const IID: TGUID; out Obj): HResult stdcall;
    function _AddRef: Integer stdcall;
    function _Release: Integer stdcall;

    // === IrUnknown ===
    function _SetOwnerInterface(aOwner: IUnknown): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastError: HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOwnerInterface(out aOwner: IUnknown): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassDisplayName(out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceCount(out aCount: Integer): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassModuleName(out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetHaveRefCount(out aHave: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Dispose(aMode: DWORD): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

    property Moniker: IrPluginMoniker read FMoniker write FMoniker;
  end;

  TrInterfacedObject = class(TInterfacedObject, IrUnknown)
  protected
    FLastError: HRESULT;
    FLastErrorMsg: string;
    FOwnerInterface: IUnknown;
    function SetError(aErrCode: HRESULT = S_OK; aExc: Exception = nil; aMsg: string = crEmptyStr): HRESULT; virtual;
  public
    function _SetOwnerInterface(aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastError: HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetOwnerInterface(out aOwner: IUnknown): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassDisplayName(out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceCount(out aCount: Integer): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetClassModuleName(out aName: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetHaveRefCount(out aHave: Boolean): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function Dispose(aMode: DWORD): HRESULT
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

  TrCustomInterfacedModule = class(TrInterfacedObject, IrCustomPlugin)
  public
    function GetOptParams(out aParams: IrMemHolder): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function SetCallback(aCallback: IUnknown): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
    function GetCallback(out aCallback: IUnknown): HRESULT; virtual;
      {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
  end;

  TrCreateInstanceProc = function (out aInstance: IUnknown): HRESULT;
    {$IFDEF MSWINDOWS} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}

function rs(const aStr: string): IrMemHolder; overload;
function rs(aHolder: IrMemHolder): string; overload;
function GetObjectErrorMessage(const aObject: IrUnknown): string;

implementation

const
  cFileMask = '*.psm';
  cCreateInstanceProcName = 'CreateInstance';
  {$ifdef mswindows}
  cPluginsRegPath = '\Plugins\DefaultTypes';
  {$endif}

type
  prPlugTypeStruct = ^TrPlugTypeStruct;
  TrPlugTypeStruct = record
    TypeID: TGUID;
    TypeName: string;
    DefaultPlugin: TrPluginLoader;
  end;

  {$ifndef mswindows}
  BOOL = LongBool;
  {$endif}

function rs(const aStr: string): IrMemHolder;
var
  h: HRESULT;
begin
  Result := nil;
  if aStr = crEmptyStr then Exit;
  h := CreateMemHolder(Result);
  if h <> S_OK then raise Exception.CreateFmt(srMemError, [h]);
  h := Result.AllocFromString(PChar(aStr));
  if h <> S_OK then raise Exception.CreateFmt(srMemError, [h]);
end;

function rs(aHolder: IrMemHolder): string;
var
  h: HRESULT;
  p: PChar;
begin
  Result := crEmptyStr;
  if aHolder = nil then Exit;
  h := aHolder.GetPtr(p);
  if h <> S_OK then raise Exception.CreateFmt(srMemError, [h]);
  if p <> nil then Result := p;
end;

function GetObjectErrorMessage(const aObject: IrUnknown): string;
var
  m: IrMemHolder;
  r: HRESULT;
begin
  if (aObject <> nil) then begin
    r := aObject.GetLastError();
    if BOOL(aObject.GetLastErrorMsg(m)) or (crEmptyStr = rs(m))
      then Result := Format(srUnknownError, [r])
      else Result := rs(m);
  end
  else
    Result := crEmptyStr;
end;

{ TrPluginLoader }

constructor TrPluginLoader.Create(aOwner: TComponent);
begin
  inherited;
  InitializeCriticalSection(FCritic);
  FSupportedTypes := TStringList.Create;
  FOptParams := TStringList.Create;
  FOptParams.Sorted := True;
  FOptParams.Duplicates := dupIgnore;
  FAutoUnload := True;
  {$ifdef linux}
  LoadMode := RTLD_LAZY or RTLD_GLOBAL;
  {$endif}
end;

function TrPluginLoader.CheckRequiredEntry: Boolean;
begin
  if not Active then Result := False
  else Result := Assigned(EntryPoint[cCreateInstanceProcName]);
end;

procedure TrPluginLoader.UpdateInfo;
var
  u: IUnknown;
  o: IrCustomPlugin;
  m: IrMemHolder;
  pr: TrCreateInstanceProc;
  b: Boolean;
  i, c: Integer;
  iid: TGUID;
  s: string;
  h: HRESULT;
  p: prPlugTypeStruct;
begin
  FSupportedTypes.Clear();
  if not (Owner is TrPluginMoniker) then Exit;
  pr := EntryPoint[cCreateInstanceProcName];
  pr(u);
{$IFDEF POLARIS_D5}
  if not Supports(u, IrCustomPlugin, o) then Exit;
{$ENDIF}
  if not BOOL(o.GetClassDisplayName(m)) then begin
    FPlugName := rs(m);
    if not BOOL(o.GetInterfaceCount(c)) then begin
      for i := 0 to c - 1 do begin
        if BOOL(o.GetInterfaceID(i, m)) then Break;
        s := rs(m);
        if TrPluginMoniker(Owner).FInvisibleTypes.IndexOf(s) >= 0 then Continue;
        iid := StringToGUID(s);
        b := TrPluginMoniker(Owner).IsKnownIID(iid, Pointer(p));
        if not b then begin
          h := o.GetInterfaceDisplayName(rs(s), m);
          b := not BOOL(h);
          if b then begin
            if rs(m) = crEmptyStr then begin
              TrPluginMoniker(Owner).FInvisibleTypes.Add(s);
              b := False;
            end
            else begin
              TrPluginMoniker(Owner).RegisterType(iid, rs(m), Self, Pointer(p));
              FSupportedTypes.Add(p.TypeName);
            end;
          end;
        end;
        if not b then Continue;
        if TrPluginMoniker(Owner).SelectPlugin(p.TypeName, FPlugName, False) = nil then begin
          TrPluginMoniker(Owner).GetTypeByTypeName(p.TypeName, Pointer(p)).Add(Self);
          FSupportedTypes.Add(p.TypeName);
        end;
        if not BOOL(o.GetOptParams(m))
          then FOptParams.Text := rs(m);
      end;
    end;
  end;
  if not BOOL(o.GetHaveRefCount(b)) and not b then begin
    DWORD(u) := 0;
    o.Dispose(1);
    DWORD(o) := 0;
  end;
end;

function TrPluginLoader.GetOptParam(aParamName: string): string;
begin
  Result := FOptParams.Values[aParamName];
end;

procedure TrPluginLoader.IncInstCount;
begin
  EnterCriticalSection(FCritic);
  Inc(FInstCount);
  LeaveCriticalSection(FCritic);
end;

procedure TrPluginLoader.DecInstCount;
begin
  EnterCriticalSection(FCritic);
  try
    Dec(FInstCount);
    if FInstCount < 1 then begin
      if FAutoUnload then Close();
      FInstCount := 0;
    end;
  finally
    LeaveCriticalSection(FCritic);
  end;
end;

function TrPluginLoader.CheckOptParam(const aParamName, aParamValue: string): Boolean;
var
  s: string;
  i: Integer;
begin
  Result := False;
  s := FOptParams.Values[aParamName];
  for i := 1 to WordCount(s, [RegStringDelim]) do begin
    Result := ExtractWord(i, s, [RegStringDelim]) = aParamValue;
    if Result then Break;
  end;
end;

destructor TrPluginLoader.Destroy;
begin
  FOptParams.Free;
  FSupportedTypes.Free;
  DeleteCriticalSection(FCritic);
  inherited;
end;

{ TrPluginMoniker }

constructor TrPluginMoniker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {$ifdef mswindows}
  FReg := TRegistry.Create;
  {$endif}
  FPlugins := TrTree.Create;
{$IFDEF POLARIS_D5}
  FPlugins.OnNotify := TreeNotify;
{$ENDIF}
  FInvisibleTypes := TStringList.Create;
  FFileMask := cFileMask;
  FAutoUnload := True;
end;

function TrPluginMoniker.SetError(aCode: HRESULT; aExc: Exception; aMsg: string): HRESULT;
begin
  FLastError := aCode;
  if aExc <> nil
    then FLastErrorMsg := aExc.Message
    else FLastErrorMsg := aMsg;
  Result := aCode;
end;

{$IFDEF POLARIS_D5}
procedure TrPluginMoniker.TreeNotify(Sender: TObject; Ptr: Pointer; Action: TListNotification);
begin
  if Action = lnDeleted then begin
    if Sender = FPlugins
      then System.Dispose(prPlugTypeStruct(Ptr))
      else with TrPluginLoader(Ptr) do begin
        FSupportedTypes.Delete(0);
        if FSupportedTypes.Count = 0 then Free();
      end;
  end;
end;
{$ENDIF}

procedure TrPluginMoniker._SetPlugsPath(const Value: string);
var
  i: Integer;
  sr: TSearchRec;
  sl: TStringList;
  sError: string;
  plug: TrPluginLoader;
  vbOpenError: Boolean;
begin

  plug := nil;
  sl := TStringList.Create;
  try
    FPlugins.Clear();
    FPlugsPath := Value;
    if FindFirst(AddSlash(FPlugsPath) + FFileMask, faAnyFile - faDirectory, sr) = 0 then
      try
        sl.Add(sr.Name);
        while FindNext(sr) = 0 do sl.Add(sr.Name);
      finally
        FindClose(sr);
      end;

    sError := crEmptyStr;
    for i := 0 to sl.Count - 1 do begin
      if plug = nil then begin
        plug := TrPluginLoader.Create(Self);
        if FRequiredVersion <> crEmptyStr then plug.RequiredVersion := FRequiredVersion;
        plug.AutoUnload := AutoUnload;
      end;
      plug.LibraryName := AddSlash(FPlugsPath) + sl.Strings[i];
      vbOpenError := False;
      try
        plug.Open;
      except
        on E: Exception do begin
          sError := sError + crlf + E.Message;
          vbOpenError := True;
        end;
      end;
      if vbOpenError and Assigned(FOnPluginLoadError) then begin
        FOnPluginLoadError(Self, plug.LibraryName, sError);
        sError := crEmptyStr;
      end;
      if not vbOpenError and plug.CheckRequiredEntry() then begin
        plug.UpdateInfo();
        plug.Close();
        plug := nil;
      end
      else plug.Close();
    end;
    plug.Free;
    plug := nil;
    RestoreSettings();

    if sError <> crEmptyStr then raise Exception.Create(sError);
  finally
    sl.Free;
    plug.Free;
  end;
end;

function TrPluginMoniker.IsKnownIID(aIID: TGUID; out aTypeDescript: Pointer): Boolean;
var
  i: Integer;
  p: prPlugTypeStruct;
begin
  Result := False;
  for i := 0 to FPlugins.NodesCount - 1 do begin
    p := FPlugins.Nodes[i];
    Result := IsEqualGUID(p.TypeID, aIID);
    if Result then begin
      aTypeDescript := p;
      Break;
    end;
  end;
end;

procedure TrPluginMoniker.RegisterType(aIID: TGUID; const aTypeName: string;
  aPlugin: TrPluginLoader; out aTypeDescript: Pointer);
var
  p: prPlugTypeStruct;
begin
  New(p);
  p.TypeID := aIID;
  p.TypeName := aTypeName;
  p.DefaultPlugin := aPlugin;
  FPlugins.AddNode(p);
  FPlugins.NodeObjects[FPlugins.NodesCount - 1].Add(aPlugin);
   aTypeDescript := p;
end;

function TrPluginMoniker.GetTypeByTypeName(aPlugType: string; var aTypeStruct: Pointer): TrList;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to FPlugins.NodesCount - 1 do begin
    aTypeStruct := FPlugins.Nodes[i];
    if prPlugTypeStruct(aTypeStruct).TypeName = aPlugType then begin
      Result := FPlugins.NodeObjects[i];
      Break;
    end;
  end;
end;

function TrPluginMoniker.SelectPlugin(const aPlugType, aPlugName: string;
  aDefaultIfNotExists: Boolean): TrPluginLoader;
  //
  function _selectplugin(aType: TrList; aName: string): TrPluginLoader;
  var
    i: Integer;
  begin
    Result := nil;
    for i := 0 to aType.Count - 1 do
      if aName = TrPluginLoader(aType[i]).PlugName then begin
        Result := TrPluginLoader(aType[i]);
        Break;
      end;
  end;
  //
var
  i: Integer;
  p: prPlugTypeStruct;
  n: TrList;
begin
  Result := nil;
  if aPlugType <> crEmptyStr then begin
    n := GetTypeByTypeName(aPlugType, Pointer(p));
    if n <> nil then begin
      if aPlugName = crEmptyStr then Result := p.DefaultPlugin
      else begin
        Result := _selectplugin(n, aPlugName);
        if (Result = nil) and aDefaultIfNotExists then Result := p.DefaultPlugin;
      end;
    end;
  end
  else begin
    if aPlugName = crEmptyStr then Exit;
    for i := 0 to FPlugins.NodesCount - 1 do begin
      Result := _selectplugin(FPlugins.NodeObjects[i], aPlugName);
      if Result <> nil then Break;
    end;
  end;
end;

function TrPluginMoniker.GetPlugTypes(const aPlugName: string): string;
var
  i: Integer;
  sl: TStringList;
  p: TrPluginLoader;
begin
  Result := crEmptyStr;
  if aPlugName <> crEmptyStr then begin
    p := SelectPlugin(crEmptyStr, aPlugName, False);
    if p <> nil then Result := p.FSupportedTypes.Text;
  end
  else begin
    sl := TStringList.Create();
    try
      for i := 0 to FPlugins.NodesCount - 1 do
        sl.Add(prPlugTypeStruct(FPlugins.Nodes[i]).TypeName);
      Result := sl.Text;
    finally
      sl.Free();
    end;
  end;
end;

function TrPluginMoniker.GetPlugNames(const aPlugType: string): string;
var
  sl: TStringList;
  //
  procedure _getplugnames(aType: TrList);
  var
    i: Integer;
  begin
    for i := 0 to aType.Count - 1 do
      sl.Add(TrPluginLoader(aType[i]).PlugName);
  end;
  //
var
  i: Integer;
  p: prPlugTypeStruct;
  n: TrList;
begin
  sl := TStringList.Create();
  try
    sl.Sorted := True;
    sl.Duplicates := dupIgnore;
    if aPlugType <> crEmptyStr then begin
      n := GetTypeByTypeName(aPlugType, Pointer(p));
      if n <> nil then
        _getplugnames(n);
    end
    else begin
      for i := 0 to FPlugins.NodesCount - 1 do
        _getplugnames(FPlugins.NodeObjects[i]);
    end;
    Result := sl.Text;
  finally
    sl.Free();
  end;
end;

function TrPluginMoniker.GetPluginByName(const aPlugName: string): TrPluginLoader;
begin
  Result := SelectPlugin(crEmptyStr, aPlugName, False);
end;

function TrPluginMoniker.GetDefaultPlugin(const aPlugType: string): TrPluginLoader;
var
  n: TrList;
  p: prPlugTypeStruct;
begin
  n := GetTypeByTypeName(aPlugType, Pointer(p));
  if n = nil
    then Result := nil
    else Result := p.DefaultPlugin;
end;

procedure TrPluginMoniker.SetDefaultPlug(const aPlugType, aPlugName: string);
var
  pl: TrPluginLoader;
  n: TrList;
  p: prPlugTypeStruct;
begin
  pl := SelectPlugin(aPlugType, aPlugName, False);
  if pl = nil then Exit;
  n := GetTypeByTypeName(aPlugType, Pointer(p));
  if n <> nil then begin
    p.DefaultPlugin := pl;
    FModifyed := True;
  end;
end;

function TrPluginMoniker.GetDefaultPlug(const aPlugType: string): string;
var
  p: TrPluginLoader;
begin
  p := GetDefaultPlugin(aPlugType);
  if p <> nil then Result := p.PlugName else Result := crEmptyStr;
end;

function TrPluginMoniker.PlugExists(const aPlugType, aPlugName: string): Boolean;
begin
  Result := SelectPlugin(aPlugType, aPlugName, False) <> nil;
end;

function TrPluginMoniker.GetPlugByOptParams(aPlugType, aParamName, aParamValue: string): string;
  //
  function _getplugbyoptparams(aType: TrList): string;
  var
    i: Integer;
  begin
    Result := crEmptyStr;
    for i := 0 to aType.Count - 1 do with TrPluginLoader(aType[i]) do
      if CheckOptParam(aParamName, aParamValue) then begin
        Result := PlugName;
        Break;
      end;
  end;
  //
var
  i: Integer;
  p: prPlugTypeStruct;
  n: TrList;
begin
  Result := crEmptyStr;
  if aPlugType <> crEmptyStr then begin
    n := GetTypeByTypeName(aPlugType, Pointer(p));
    if n <> nil then
      Result := _getplugbyoptparams(n);
  end
  else begin
    for i := 0 to FPlugins.NodesCount - 1 do begin
      Result := _getplugbyoptparams(FPlugins.NodeObjects[i]);
      if Result <> crEmptyStr then Break;
    end;
  end;
end;

function TrPluginMoniker.CheckVersionFormat: Boolean;
var
  MS, LS: DWORD;
begin
  Result := SeparateVersion(fRequiredVersion, MS, LS);
end;

procedure TrPluginMoniker.SetRequiredVersion(const Value: string);
var
  MS, LS: DWORD;
begin
  if SeparateVersion(Value, MS, LS) then fRequiredVersion := Value;
end;

function TrPluginMoniker.GetCount: Integer;
begin
  Result := FPlugins.Count;
end;

function TrPluginMoniker.GetLastError: HRESULT;
begin
  Result := FLastError;
end;

function TrPluginMoniker.GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT;
begin
  Result := S_OK;
  try
{$ifdef ps_nativestringutf}
    aMsg := rs(AnsiToUtf8(FLastErrorMsg));
{$else}
    aMsg := rs(FLastErrorMsg);
{$endif}
  except
    Result := PE_rUnknown_GetLastErrorMsg;
  end;
end;

function TrPluginMoniker._SetOwnerInterface(aOwner: IUnknown): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginMoniker.GetOwnerInterface(out aOwner: IUnknown): HRESULT;
begin
  if Owner <> nil then
    aOwner := Owner as IUnknown;
  Result := SetError();
end;

function TrPluginMoniker.GetClassDisplayName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    rs('Plugin moniker component');
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetClassDisplayName, E);
  end;
end;

function TrPluginMoniker.GetClassModuleName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
{$ifdef ps_nativestringutf}
    aName := rs(AnsiToUtf8(GetModulePath(hInstance)));
{$else}
    aName := rs(GetModulePath(hInstance));
{$endif}
  except
    Result := PE_rUnknown_GetClassModuleName;
  end;
end;

function TrPluginMoniker.GetHaveRefCount(out aHave: Boolean): HRESULT;
begin
  aHave := False;
  Result := SetError();
end;

function TrPluginMoniker.GetInterfaceCount(out aCount: Integer): HRESULT;
begin
  aCount := GetInterfaceTable.EntryCount;
  Result := SetError();
end;

function TrPluginMoniker.GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    if aNumber > GetInterfaceTable.EntryCount - 1
      then Result := SetError(PE_rUnknown_GetInterfaceID)
      else aID := rs(GUIDToString(GetInterfaceTable.Entries[aNumber].IID));
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetInterfaceID, E);
  end;
end;

function TrPluginMoniker.GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT;
var
  s: string;
  p: prPlugTypeStruct;
begin
  Result := SetError();
  try
    s := rs(aID);
    if s = crEmptyStr then Exit;
    if IsKnownIID(StringToGUID(s), Pointer(p)) then
{$ifdef ps_nativestringutf}
      aName := rs(AnsiToUtf8(p.TypeName));
{$else}
      aName := rs(p.TypeName);
{$endif}
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetInterfaceDisplayName, E);
  end;
end;

function TrPluginMoniker.Dispose(aMode: DWORD): HRESULT;
begin
  Result := PE_rUnknown_CapabilityNotSupported;
end;

function TrPluginMoniker._DecInstanceCounter(aObject: IrUnknown): HRESULT;
var
  p: TrPluginLoader;
  i: IrMemHolder;
begin
  if aObject = nil then begin
    Result := SetError(PE_rPluginMoniker_UnknownPlugin);
    Exit;
  end;
  try
    Result := SetError(aObject.GetClassDisplayName(i));
    if BOOL(Result) then Exit;
    Result := SetError(aObject.Dispose(1));
    DWORD(aObject) := 0;
    p := GetPluginByName(rs(i));
    if p = nil
      then Result := SetError(PE_rPluginMoniker_UnknownPlugin)
      else p.DecInstCount();
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_DecInstanceCounter, E);
  end;
end;

function TrPluginMoniker.NewObjectInstance(aClassName, aInterfaceName: IrMemHolder;
  aDefaultIfNotExists: Boolean; out aObject: IrUnknown): HRESULT;
var
  sPlugType, sPlugName: string;
  plug: TrPluginLoader;
  proc: TrCreateInstanceProc;
  iu: IUnknown;
  iru: IrUnknown;
  ip: IrCustomPlugin;
  p: prPlugTypeStruct;
begin
  Result := SetError();
  try
{$ifdef ps_nativestringutf}
    sPlugType := Utf8ToAnsi(rs(aInterfaceName));
    sPlugName := Utf8ToAnsi(rs(aClassName));
{$else}
    sPlugType := rs(aInterfaceName);
    sPlugName := rs(aClassName);
{$endif}
    if (sPlugType = crEmptyStr) and (sPlugName = crEmptyStr) then begin
      Result := SetError(PE_rPluginMoniker_BadParams, nil, srBadParams);
      Exit;
    end;
    plug := SelectPlugin(sPlugType, sPlugName, aDefaultIfNotExists);
    if plug = nil then begin
      aObject := nil;
      Result := SetError(PE_rPluginMoniker_UnknownPlugin, nil, Format(srPluginNotFound, [sPlugType, sPlugName]));
      Exit;
    end;
    if not plug.Active then plug.Open;
    plug.IncInstCount;
    proc := plug.EntryPoint[cCreateInstanceProcName];
    if Assigned(proc)
      then proc(iu)
      else plug.DecInstCount;
    if not Assigned(iu) then begin
      Result := SetError(PE_rCustomMoniker_NewObjectInstance, nil, Format(srIFaceNotSupported, ['IUnknown']));
      Exit;
    end;
    if not Supports(iu, IrUnknown, iru) then begin
      Result := SetError(PE_rCustomMoniker_NewObjectInstance, nil, Format(srIFaceNotSupported, ['IrUnknown']));
      Exit;
    end;
    iru._SetOwnerInterface(Self as IUnknown);
    if (FCallback <> nil) and Supports(iru, IrCustomPlugin, ip) then begin
      ip.SetCallback(FCallback);
      ip := nil;
    end;
    if sPlugType = crEmptyStr then aObject := iru
    else begin
      if GetTypeByTypeName(sPlugType, Pointer(p)) = nil then begin
        Result := SetError(PE_rUnknown_UnknownInterface, nil, Format(srNoInterface, [sPlugType]));
        rDispose(iru);
        Exit;
      end;
      if not Supports(iru, p.TypeID, aObject) then begin
        Result := SetError(PE_rUnknown_UnknownInterface, nil, Format(srIFaceNotSupported, [sPlugType]));
        rDispose(iru);
        Exit;
      end;
    end;
  except
    on E: Exception do begin
      rDispose(iru);
      Result := SetError(PE_rCustomMoniker_NewObjectInstance, E);
    end;
  end;
end;

function TrPluginMoniker.GetClassesCount(out aCount: Integer): HRESULT;
begin
  Result := SetError();
  try
    aCount := FPluginCount;
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetClassesCount, E);
  end;
end;

function TrPluginMoniker.GetClassInterfaces(aClassName: IrMemHolder; out aInterfaceNames: IrMemHolder): HRESULT;
var
  p: TrPluginLoader;
begin
  Result := SetError();
  try
    p := GetPluginByName(rs(aClassName));
    if p = nil
      then aInterfaceNames := rs(GetPlugTypes())
      else aInterfaceNames := rs(p.FSupportedTypes.Text);
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetClassInterfaces, E);
  end;
end;

function TrPluginMoniker.GetClassNames(aInterfaceName: IrMemHolder; out aClassNames: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    aClassNames := rs(GetPlugNames(rs(aInterfaceName)));
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetClassNames, E);
  end;
end;

function TrPluginMoniker.GetDefaultClassName(aInterfaceName: IrMemHolder; out aClassName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    aClassName := rs(GetDefaultPlug(rs(aInterfaceName)));
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetDefaultClassName, E);
  end;
end;

function TrPluginMoniker.SetDefaultClassName(aInterfaceName, aClassName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    SetDefaultPlug(rs(aInterfaceName), rs(aClassName));
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetDefaultClassName, E);
  end;
end;

function TrPluginMoniker.GetInterfaceByDisplayName(aName: IrMemHolder; out aID: IrMemHolder): HRESULT;
var
  i: Integer;
  p: prPlugTypeStruct;
  s: string;
begin
  Result := SetError(PE_rUnknown_UnknownInterface);
  try
{$ifdef ps_nativestringutf}
    s := Utf8ToAnsi(rs(aName));
{$else}
    s := rs(aName);
{$endif}
    for i := 0 to FPlugins.NodesCount - 1 do begin
      p := prPlugTypeStruct(FPlugins.Nodes[i]);
      if p.TypeName = s then begin
        aID := rs(GUIDToString(p.TypeID));
        Result := SetError(S_OK);
        Break;
      end;
    end;
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetInterfaceByDisplayName, E);
  end;
end;

function TrPluginMoniker.GetPlugByOptParam(aPlugType, aParamName,
  aParamValue: IrMemHolder; out aPlugName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    aPlugName :=
{$ifdef ps_nativestringutf}
      rs(AnsiToUtf8(GetPlugByOptParams(
        Utf8ToAnsi(rs(aPlugType)), Utf8ToAnsi(rs(aParamName)), Utf8ToAnsi(rs(aParamValue)))));
{$else}
      rs(GetPlugByOptParams(
        rs(aPlugType), rs(aParamName), rs(aParamValue)));
{$endif}
  except
    on E: Exception do Result := SetError(PE_rPluginMoniker_GetPlugByOptParam, E);
  end;
end;

function TrPluginMoniker.GetOptParam(aPlugName, aParamName: IrMemHolder; out aValue: IrMemHolder): HRESULT;
var
  pl: TrPluginLoader;
  s: string;
begin
  Result := SetError();
  try
{$ifdef ps_nativestringutf}
    s := Utf8ToAnsi(rs(aPlugName));
{$else}
    s := rs(aPlugName);
{$endif}
    pl := SelectPlugin(crEmptyStr, s, False);
    if pl = nil then
      SetError(PE_rPluginMoniker_GetPlugByOptParam, nil, Format(srPluginNotFound, [crEmptyStr, s]))
    else
{$ifdef ps_nativestringutf}
      aValue := rs(AnsiToUtf8(pl.OptParam[rs(aParamName)]));
{$else}
      aValue := rs(pl.OptParam[rs(aParamName)]);
{$endif}
  except
    on E: Exception do Result := SetError(PE_rPluginMoniker_GetPlugByOptParam, E);
  end;
end;

function TrPluginMoniker.GetActiveObjectsCount(out aCount: Integer): HRESULT;
var
  i, j: Integer;
  n: TrTree;
begin
  Result := SetError();
  try
    aCount := 0;
    for i := 0 to FPlugins.NodesCount - 1 do begin
      n := FPlugins.NodeObjects[i];
      for j := 0 to n.Count - 1 do
        if TrPluginLoader(n.Items[j]).Active then Inc(aCount);
    end;
  except
    on E: Exception do Result := SetError(PE_rCustomMoniker_GetActiveObjectsCount, E);
  end;
end;

function TrPluginMoniker.GetPlugsPath(out aPath: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    aPath := rs(FPlugsPath);
  except
    on E: Exception do Result := SetError(PE_rPluginMoniker_GetPlugsPath, E);
  end;
end;

function TrPluginMoniker.SetPlugsPath(aPath: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    _SetPlugsPath(rs(aPath));
  except
    on E: Exception do Result := SetError(PE_rPluginMoniker_SetPlugsPath, E);
  end;
end;

function TrPluginMoniker.RestoreSettings(): HRESULT;
{$ifdef mswindows}
var
  r: HKEY;
  s: string;
  i: Integer;
{$endif}
begin
  Result := SetError();
  try
    {$ifdef mswindows}
    if FRegKey <> crEmptyStr then begin
      SeparateRegKey(FRegKey, r, s);
      FReg.RootKey := r;
      FReg.OpenKey(s + cPluginsRegPath, True);
      try
        for i := 0 to FPlugins.NodesCount - 1 do begin
          s := prPlugTypeStruct(FPlugins.Nodes[i]).TypeName;
          if FReg.ValueExists(s)
            then SetDefaultPlug(s, FReg.ReadString(s));
        end;
      finally
        FReg.CloseKey;
      end;
    end;
    {$endif}
    FModifyed := False;
  except
    on E: Exception do Result := SetError(PE_rPluginMoniker_RestoreSettings, E);
  end;
end;

function TrPluginMoniker.StoreSettings(): HRESULT;
{$ifdef mswindows}
var
  i: Integer;
  s: string;
  r: HKEY;
  p: prPlugTypeStruct;
{$endif}
begin
  Result := SetError();
  if not FModifyed then Exit;
  try
    {$ifdef mswindows}
    if FRegKey <> crEmptyStr then begin
      SeparateRegKey(FRegKey, r, s);
      FReg.RootKey := r;
      FReg.OpenKey(s + cPluginsRegPath, True);
      try
        for i := 0 to FPlugins.NodesCount - 1 do begin
          p := prPlugTypeStruct(FPlugins.Nodes[i]);
          FReg.WriteString(p.TypeName, p.DefaultPlugin.PlugName);
        end;
      finally
        FReg.CloseKey;
      end;
    end;
    {$endif}
    FModifyed := False;
  except
    on E: Exception do Result := SetError(PE_rPluginMoniker_StoreSettings, E);
  end;
end;

destructor TrPluginMoniker.Destroy;
begin
  StoreSettings;
  FPlugins.Free;
  FInvisibleTypes.Free;
  {$ifdef mswindows}
  FReg.Free;
  {$endif}
  inherited Destroy;
end;

function TrPluginMoniker.GetPlugTypeByGUID(aGUID: TGUID): string;
var
  i: Integer;
begin
  Result := crEmptyStr;
  for i := 0 to FPlugins.NodesCount - 1 do begin
    with prPlugTypeStruct(FPlugins.Nodes[i])^ do
      if GUIDToString(TypeID) = GUIDToString(aGUID) then begin
        Result := TypeName;
        Break;
      end;
  end;
end;

{ TrCustomPlugin }

constructor TrCustomPlugin.Create;
begin
  inherited;
  FModuleName := GetModulePath(hInstance);
end;

function TrCustomPlugin.SetError(aErrCode: HRESULT; aExc: Exception; aMsg: string): HRESULT;
begin
  FLastError := aErrCode;
  if aExc <> nil then begin
    if aExc is ErPlugBaseError
      then FLastErrorMsg := aExc.Message
      else FLastErrorMsg := Format(srUnkErrorDesc, [aExc.ClassName, aExc.Message]);
  end else
    if aMsg = crEmptyStr then
      if FLastError = S_OK then
        FLastErrorMsg := crEmptyStr
      else
        FLastErrorMsg := Format(srUnkErrorDesc, [IntToStr(aErrCode), '( )'])
    else
      FLastErrorMsg := aMsg;
  Result := aErrCode;
end;

function TrCustomPlugin.QueryInterface(const IID: TGUID; out Obj): HRESULT;
begin
  if GetInterface(IID, Obj)
    then Result := S_OK
    else Result := E_NOINTERFACE;
end;

function TrCustomPlugin._AddRef: Integer;
begin
{$ifdef ps_nopluginrefcount}
  Result := -1;
{$else}
  Result := InterlockedIncrement(FRefCount);
{$endif}
end;

function TrCustomPlugin._Release: Integer;
{$if not defined(ps_nopluginrefcount) and defined(mswindows)}
var
  p: Pointer;
{$ifend}
begin
{$ifdef ps_nopluginrefcount}
  Result := -1;
{$else}
  Result := InterlockedDecrement(FRefCount);
  if Result = 0 then begin
  {$ifdef mswindows}
    p := @FReleaseData;
    asm
      jmp @@_release_2
@@_release_1:
      push  0
      push  dword ptr [eax]
      add   eax, 4
      push  dword ptr [eax]
      jmp   Dispose
@@_release_2:
      mov  ecx, p
      mov  eax, Self
      mov  dword ptr [ecx], eax
      lea  eax, Self
      sub  eax, 4
      mov  edx, dword ptr [eax]
      add  ecx, 4
      mov  dword ptr [ecx], edx
      lea  ecx, @@_release_1
      mov  dword ptr [eax], ecx
    end;
    Result := Integer(p);
  {$endif}
  {$ifdef linux}
    Destroy;
  {$endif}
  end;
{$endif}
end;

function TrCustomPlugin._SetOwnerInterface(aOwner: IInterface): HRESULT;
var
  m: IrMemHolder;
begin
  try
    FClassDisplayName := crEmptyStr;
    if Supports(aOwner, IrPluginMoniker, FMoniker) then begin
      Result := GetClassDisplayName(m);
      if not BOOL(Result) then FClassDisplayName := rs(m);
    end
    else Result := SetError(PE_rUnknown_UnknownInterface, nil, Format(srIFaceNotSupported, ['IrPluginMoniker']));
  except
    on E: Exception do Result := SetError(PE_rUnknown_SetOwnerInterface, E);
  end;
end;

function TrCustomPlugin.GetLastError: HRESULT;
begin
  Result := FLastError;
end;

function TrCustomPlugin.GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT;
begin
  Result := S_OK;
  try
{$ifdef ps_nativestringutf}
    aMsg := rs(AnsiToUtf8(FLastErrorMsg));
{$else}
    aMsg := rs(FLastErrorMsg);
{$endif}
  except
    Result := PE_rUnknown_GetLastErrorMsg;
  end;
end;

function TrCustomPlugin.GetOwnerInterface(out aOwner: IInterface): HRESULT;
begin
  aOwner := FMoniker;
  Result := SetError();
end;

function TrCustomPlugin.GetClassModuleName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
{$ifdef ps_nativestringutf}
    aName := rs(AnsiToUtf8(FModuleName));
{$else}
    aName := rs(FModuleName);
{$endif}
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetClassModuleName, E);
  end;
end;

function TrCustomPlugin.GetInterfaceCount(out aCount: Integer): HRESULT;
begin
  aCount := GetInterfaceTable.EntryCount;
  Result := SetError();
end;

function TrCustomPlugin.GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    if aNumber > GetInterfaceTable.EntryCount - 1
      then Result := SetError(PE_rUnknown_GetInterfaceID)
      else aID := rs(GUIDToString(GetInterfaceTable.Entries[aNumber].IID));
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetInterfaceID, E);
  end;
end;

function TrCustomPlugin.GetHaveRefCount(out aHave: Boolean): HRESULT;
begin
  Result := SetError();
{$ifdef ps_nopluginrefcount}
  aHave := False;
{$else}
  aHave := True;
{$endif}
end;

function TrCustomPlugin.QueryPlugin(aInterfaceName, aClassName: string;
  aDefaultIfNotExists, aRaiseIfNotFound: Boolean; out aObject): HRESULT;
begin
  if FMoniker <> nil then begin
    Result := FMoniker.NewObjectInstance(
      rs(aClassName), rs(aInterfaceName), aDefaultIfNotExists, IrUnknown(aObject));
    if BOOL(Result) and aRaiseIfNotFound then
      raise ErPlugBaseError.Create(GetObjectErrorMessage(FMoniker));
    if not BOOL(Result) and (FCallback <> nil) then IrCustomPlugin(aObject).SetCallback(FCallback);
  end
  else begin
    if aRaiseIfNotFound then
      raise ErPlugBaseError.CreateFmt(srNoInterface, ['IrPluginMoniker']);
    Result := SetError(PE_rUnknown_UnknownInterface, nil, Format(srNoInterface, ['IrPluginMoniker']));
  end;
end;

function TrCustomPlugin.CallPlugin(aResult: HRESULT; aPlugin: IrUnknown): HRESULT;
var
  i: IrMemHolder;
begin
  Result := SetError(aResult);
  if BOOL(aResult) and (aPlugin <> nil) and not BOOL(aPlugin.GetLastErrorMsg(i))
    then FLastErrorMsg := rs(i);
end;

function TrCustomPlugin.QueryCallback(aIID: TGUID; out aIntf): HRESULT;
var
  u: IUnknown;
begin
  Result := PE_rUnknown_UnknownInterface;
  FLastErrorMsg := crEmptyStr;
  if FCallback <> nil then begin
    if Supports(FCallback, aIID, aIntf) then Result := S_OK;
  end else
  if FMoniker <> nil then begin
    if Supports(FMoniker, aIID, aIntf) then Result := S_OK
    else begin
      if not BOOL(FMoniker.GetOwnerInterface(u)) and Supports(u, aIID, aIntf)
        then Result := S_OK;
    end;
  end;
end;

function TrCustomPlugin.SetCallback(aCallback: IUnknown): HRESULT;
begin
  Result := SetError();
  FCallback := aCallback;
end;

function TrCustomPlugin.GetCallback(out aCallback: IUnknown): HRESULT;
begin
  aCallback := FCallback;
  Result := SetError();
end;

function TrCustomPlugin.__SafeFree(): HRESULT;
begin
  Result := S_OK;
  try
    Self.Free();
  except
    Result := PE_rUnknown_Dispose;
  end;
end;

function Get_rUnknown_GUID: TGUID;
begin
  Result := IrUnknown;
end;

function TrCustomPlugin.Dispose(aMode: DWORD): HRESULT;
{pascal equivalent:
var
  i: DWORD;
begin
  if BOOL(aMode and 1) or (FMoniker = nil)
    then Result := __SafeFree()
    else begin
      GetInterface(Get_rUnknown_GUID(), i);
      Result := FMoniker._DecInstanceCounter(IrUnknown(i));
      // Note !!!
      // During a call _DecInstanceCounter the plugin module may be unloaded
    end;
end;
}
asm
  // compiler added:
  // push  ebp
  // mov  ebp, esp

  add  esp, -$14        // 2 variables - DWORD & TGUID
  mov  eax, [ebp + $0c] // aMode
  bt   eax, 0
  jc   @@norm_exit
  mov  eax, [ebp + $08] // Self
  cmp  dword ptr [eax + FMoniker], 0
  jz   @@norm_exit

  lea  eax, [ebp - $14] // TGUID variable
  call Get_rUnknown_GUID
  lea  edx, [ebp - $14] // TGUID variable
  lea  ecx, [ebp - $04] // DWORD variable
  mov  eax, [ebp + $08] // Self
  call TObject.GetInterface
  mov  eax, [ebp - $04] // DWORD variable
  mov  [ebp + $0C], eax // aObject parameter for _DecInstanceCounter
  mov  eax, [ebp + $08] // Self
  mov  eax, [eax + FMoniker]
  mov  [ebp + $08], eax // FMoniker = Self parameter for _DecInstanceCounter
  mov  eax, [eax]       // interface table address
  mov  esp, ebp
  pop  ebp
  jmp  dword ptr [eax + VMTOFFSET IrPluginMoniker._DecInstanceCounter]

@@norm_exit:
  mov  eax, [ebp + $08] // Self
  call TrCustomPlugin.__SafeFree
  mov  esp, ebp

  // compiler added:
  // pop  ebp
  // ret $0008 for stdcall or ret for cdecl
end;

{ TrPluginThread }

function TrPluginThread.SetError(aCode: HRESULT; aExc: Exception; aMsg: string): HRESULT;
begin
  FLastError := aCode;
  if aExc <> nil
    then FLastErrorMsg := aExc.Message
    else FLastErrorMsg := aMsg;
  Result := aCode;
end;

function TrPluginThread.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID, Obj)
    then Result := S_OK
    else Result := E_NOINTERFACE;
end;

function TrPluginThread._AddRef: Integer;
begin
  Result := -1;
end;

function TrPluginThread._Release: Integer;
begin
  Result := -1;
end;

function TrPluginThread._SetOwnerInterface(aOwner: IInterface): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.GetLastError: HRESULT;
begin
  Result := FLastError;
end;

function TrPluginThread.GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT;
begin
  Result := S_OK;
  try
{$ifdef ps_nativestringutf}
    aMsg := rs(AnsiToUtf8(FLastErrorMsg));
{$else}
    aMsg := rs(FLastErrorMsg);
{$endif}
  except
    Result := PE_rUnknown_GetLastErrorMsg;
  end;
end;

function TrPluginThread.GetClassDisplayName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.GetClassModuleName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
{$ifdef ps_nativestringutf}
    aName := rs(AnsiToUtf8(GetModulePath(hInstance)));
{$else}
    aName := rs(GetModulePath(hInstance));
{$endif}
  except
    Result := PE_rUnknown_GetClassModuleName;
  end;
end;

function TrPluginThread.GetHaveRefCount(out aHave: Boolean): HRESULT;
begin
  Result := SetError();
  aHave := False;
end;

function TrPluginThread.GetInterfaceCount(out aCount: Integer): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.GetOwnerInterface(out aOwner: IInterface): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.Dispose(aMode: DWORD): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrPluginThread.QueryPlugin(aInterfaceName, aClassName: string;
  aDefaultIfNotExists, aRaiseIfNotFound: Boolean; out aObject: IrUnknown): HRESULT;
begin
  if FMoniker <> nil then begin
    Result := FMoniker.NewObjectInstance(rs(aClassName), rs(aInterfaceName), aDefaultIfNotExists, aObject);
    if BOOL(Result) and aRaiseIfNotFound then
      raise ErPlugBaseError.Create(GetObjectErrorMessage(FMoniker));
    if not BOOL(Result) then IrCustomPlugin(aObject).SetCallback(Self);
  end
  else begin
    if aRaiseIfNotFound then
      raise ErPlugBaseError.CreateFmt(srNoInterface, ['IrPluginMoniker']);
    Result := SetError(PE_rUnknown_UnknownInterface, nil, Format(srNoInterface, ['IrPluginMoniker']));
  end;
end;

function TrPluginThread.CallPlugin(aResult: HRESULT; aPlugin: IrCustomPlugin): HRESULT;
var
  m: IrMemHolder;
begin
  FLastError := aResult;
  if BOOL(FLastError) and (aPlugin <> nil) and not BOOL(aPlugin.GetLastErrorMsg(m))
    then FLastErrorMsg := rs(m)
    else FLastErrorMsg := crEmptyStr;
  Result := FLastError;
end;

{ TrInterfacedObject }

function TrInterfacedObject.SetError(aErrCode: HRESULT; aExc: Exception; aMsg: string): HRESULT;
begin
  FLastError := aErrCode;
  if aExc <> nil
    then FLastErrorMsg := aExc.Message
    else FLastErrorMsg := aMsg;
  Result := aErrCode;
end;

function TrInterfacedObject._SetOwnerInterface(aOwner: IUnknown): HRESULT;
begin
  Result := SetError();
  try
    FOwnerInterface := aOwner;
  except
    on E: Exception do Result := SetError(PE_rUnknown_SetOwnerInterface, E);
  end;
end;

function TrInterfacedObject.GetLastError: HRESULT;
begin
  Result := FLastError;
end;

function TrInterfacedObject.GetLastErrorMsg(out aMsg: IrMemHolder): HRESULT;
begin
  Result := S_OK;
  try
{$ifdef ps_nativestringutf}
    aMsg := rs(AnsiToUtf8(FLastErrorMsg));
{$else}
    aMsg := rs(FLastErrorMsg);
{$endif}
  except
    Result := PE_rUnknown_GetLastErrorMsg;
  end;
end;

function TrInterfacedObject.GetClassDisplayName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrInterfacedObject.GetClassModuleName(out aName: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
{$ifdef ps_nativestringutf}
    aName := rs(AnsiToUtf8(GetModulePath(hInstance)));
{$else}
    aName := rs(GetModulePath(hInstance));
{$endif}
  except
    Result := PE_rUnknown_GetClassModuleName;
  end;
end;

function TrInterfacedObject.GetHaveRefCount(out aHave: Boolean): HRESULT;
begin
  Result := SetError();
  aHave := True;
end;

function TrInterfacedObject.GetInterfaceCount(out aCount: Integer): HRESULT;
begin
  Result := SetError();
  aCount := GetInterfaceTable.EntryCount;
end;

function TrInterfacedObject.GetInterfaceDisplayName(aID: IrMemHolder; out aName: IrMemHolder): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrInterfacedObject.GetInterfaceID(aNumber: Integer; out aID: IrMemHolder): HRESULT;
begin
  Result := SetError();
  try
    if aNumber > GetInterfaceTable.EntryCount - 1
      then Result := SetError(PE_rUnknown_GetInterfaceID)
      else aID := rs(GUIDToString(GetInterfaceTable.Entries[aNumber].IID));
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetInterfaceID, E);
  end;
end;

function TrInterfacedObject.GetOwnerInterface(out aOwner: IUnknown): HRESULT;
begin
  Result := SetError();
  try
    aOwner := FOwnerInterface;
  except
    on E: Exception do Result := SetError(PE_rUnknown_GetOwnerInterface, E);
  end;
end;

function TrInterfacedObject.Dispose(aMode: DWORD): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

{ TrCustomInterfacedModule }

function TrCustomInterfacedModule.GetOptParams(out aParams: IrMemHolder): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrCustomInterfacedModule.GetCallback(out aCallback: IUnknown): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

function TrCustomInterfacedModule.SetCallback(aCallback: IUnknown): HRESULT;
begin
  Result := SetError(PE_rUnknown_CapabilityNotSupported, nil, srCapabilityNotSupported);
end;

end.

